@@ -1,12 +1,12 @@
 from autoarray.plot.abstract_plotters import Plotter
-from autoarray.plot.mat_wrap.visuals import Visuals1D
-from autoarray.plot.mat_wrap.visuals import Visuals2D
-from autoarray.plot.mat_wrap.include import Include1D
-from autoarray.plot.mat_wrap.include import Include2D
-from autoarray.plot.mat_wrap.mat_plot import MatPlot1D
-from autoarray.plot.mat_wrap.mat_plot import MatPlot2D
-from autoarray.plot.mat_wrap.mat_plot import AutoLabels
-from autoarray.dataset.interferometer import Interferometer
+from autoarray.plot.visuals.one_d import Visuals1D
+from autoarray.plot.visuals.two_d import Visuals2D
+from autoarray.plot.include.one_d import Include1D
+from autoarray.plot.include.two_d import Include2D
+from autoarray.plot.mat_plot.one_d import MatPlot1D
+from autoarray.plot.mat_plot.two_d import MatPlot2D
+from autoarray.plot.auto_labels import AutoLabels
+from autoarray.dataset.interferometer.interferometer import Interferometer
 from autoarray.structures.grids.irregular_2d import Grid2DIrregular
 
 
@@ -82,31 +82,31 @@ class InterferometerPlotter(Plotter):
         """
         Plots the individual attributes of the plotter's `Interferometer` object in 1D and 2D.
 
-        The API is such that every plottable attribute of the `Interferometer` object is an input parameter of type 
+        The API is such that every plottable attribute of the `Interferometer` object is an input parameter of type
         bool of the function, which if switched to `True` means that it is plotted.
 
         Parameters
         ----------
         visibilities
-            Whether or not to make a 2D plot (via `scatter`) of the visibility data.
+            Whether to make a 2D plot (via `scatter`) of the visibility data.
         noise_map
-            Whether or not to make a 2D plot (via `scatter`) of the noise-map.
+            Whether to make a 2D plot (via `scatter`) of the noise-map.
         u_wavelengths
-            Whether or not to make a 1D plot (via `plot`) of the u-wavelengths.          
+            Whether to make a 1D plot (via `plot`) of the u-wavelengths.
         v_wavelengths
-            Whether or not to make a 1D plot (via `plot`) of the v-wavelengths.      
+            Whether to make a 1D plot (via `plot`) of the v-wavelengths.
         amplitudes_vs_uv_distances
-            Whether or not to make a 1D plot (via `plot`) of the amplitudes versis the uv distances.   
+            Whether to make a 1D plot (via `plot`) of the amplitudes versis the uv distances.
         phases_vs_uv_distances
-            Whether or not to make a 1D plot (via `plot`) of the phases versis the uv distances.
+            Whether to make a 1D plot (via `plot`) of the phases versis the uv distances.
         dirty_image
-            Whether or not to make a 2D plot (via `imshow`) of the dirty image.
+            Whether to make a 2D plot (via `imshow`) of the dirty image.
         dirty_noise_map
-            Whether or not to make a 2D plot (via `imshow`) of the dirty noise map.
+            Whether to make a 2D plot (via `imshow`) of the dirty noise map.
         dirty_signal_to_noise_map
-            Whether or not to make a 2D plot (via `imshow`) of the dirty signal-to-noise map.
+            Whether to make a 2D plot (via `imshow`) of the dirty signal-to-noise map.
         dirty_inverse_noise_map
-            Whether or not to make a 2D plot (via `imshow`) of the dirty inverse noise map.
+            Whether to make a 2D plot (via `imshow`) of the dirty inverse noise map.
         """
 
         if visibilities:
@@ -158,8 +158,8 @@ class InterferometerPlotter(Plotter):
 
             self.mat_plot_2d.plot_grid(
                 grid=Grid2DIrregular.from_yx_1d(
-                    y=self.interferometer.uv_wavelengths[:, 1] / 10 ** 3.0,
-                    x=self.interferometer.uv_wavelengths[:, 0] / 10 ** 3.0,
+                    y=self.interferometer.uv_wavelengths[:, 1] / 10**3.0,
+                    x=self.interferometer.uv_wavelengths[:, 0] / 10**3.0,
                 ),
                 visuals_2d=self.visuals_2d,
                 auto_labels=AutoLabels(
@@ -171,7 +171,7 @@ class InterferometerPlotter(Plotter):
 
             self.mat_plot_1d.plot_yx(
                 y=self.interferometer.amplitudes,
-                x=self.interferometer.uv_distances / 10 ** 3.0,
+                x=self.interferometer.uv_distances / 10**3.0,
                 visuals_1d=self.visuals_1d,
                 auto_labels=AutoLabels(
                     title="Amplitudes vs UV-distances",
@@ -186,7 +186,7 @@ class InterferometerPlotter(Plotter):
 
             self.mat_plot_1d.plot_yx(
                 y=self.interferometer.phases,
-                x=self.interferometer.uv_distances / 10 ** 3.0,
+                x=self.interferometer.uv_distances / 10**3.0,
                 visuals_1d=self.visuals_1d,
                 auto_labels=AutoLabels(
                     title="Phases vs UV-distances",
@@ -255,7 +255,7 @@ class InterferometerPlotter(Plotter):
         """
         Plots the individual attributes of the plotter's `Interferometer` object in 1D and 2D on a subplot.
 
-        The API is such that every plottable attribute of the `Interferometer` object is an input parameter of type 
+        The API is such that every plottable attribute of the `Interferometer` object is an input parameter of type
         bool of the function, which if switched to `True` means that it is included on the subplot.
 
         Parameters
@@ -265,11 +265,11 @@ class InterferometerPlotter(Plotter):
         noise_map
             Whether or not to include a 2D plot (via `scatter`) of the noise-map.
         u_wavelengths
-            Whether or not to include a 1D plot (via `plot`) of the u-wavelengths.          
+            Whether or not to include a 1D plot (via `plot`) of the u-wavelengths.
         v_wavelengths
-            Whether or not to include a 1D plot (via `plot`) of the v-wavelengths.      
+            Whether or not to include a 1D plot (via `plot`) of the v-wavelengths.
         amplitudes_vs_uv_distances
-            Whether or not to include a 1D plot (via `plot`) of the amplitudes versis the uv distances.   
+            Whether or not to include a 1D plot (via `plot`) of the amplitudes versis the uv distances.
         phases_vs_uv_distances
             Whether or not to include a 1D plot (via `plot`) of the phases versis the uv distances.
         dirty_image
