@@ -11,7 +11,7 @@ def residual_map_from(*, data: Structure, model_data: Structure) -> Structure:
     Residuals = (Data - Model_Data).
 
     Parameters
-    -----------
+    ----------
     data
         The data that is fitted.
     mask
@@ -31,9 +31,9 @@ def normalized_residual_map_from(
     Normalized_Residual = (Data - Model_Data) / Noise
 
     Parameters
-    -----------
+    ----------
     residual_map
-        The residual-map of the model-simulator fit to the dataset.
+        The residual-map of the model-data fit to the dataset.
     noise_map
         The noise-map of the dataset.
     mask
@@ -49,9 +49,9 @@ def chi_squared_map_from(*, residual_map: Structure, noise_map: Structure) -> St
     Chi_Squared = ((Residuals) / (Noise)) ** 2.0 = ((Data - Model)**2.0)/(Variances)
 
     Parameters
-    -----------
+    ----------
     residual_map
-        The residual-map of the model-simulator fit to the dataset.
+        The residual-map of the model-data fit to the dataset.
     noise_map
         The noise-map of the dataset.
     """
@@ -67,7 +67,7 @@ def chi_squared_from(*, chi_squared_map: Structure) -> float:
     Parameters
     ----------
     chi_squared_map
-        The chi-squared-map of values of the model-simulator fit to the dataset.
+        The chi-squared-map of values of the model-data fit to the dataset.
     """
     return float(np.sum(chi_squared_map))
 
@@ -83,7 +83,7 @@ def noise_normalization_from(*, noise_map: Structure) -> float:
     noise_map
         The masked noise-map of the dataset.
     """
-    return float(np.sum(np.log(2 * np.pi * noise_map ** 2.0)))
+    return float(np.sum(np.log(2 * np.pi * noise_map**2.0)))
 
 
 def normalized_residual_map_complex_from(
@@ -95,9 +95,9 @@ def normalized_residual_map_complex_from(
     Normalized_Residual = (Data - Model_Data) / Noise
 
     Parameters
-    -----------
+    ----------
     residual_map
-        The residual-map of the model-simulator fit to the dataset.
+        The residual-map of the model-data fit to the dataset.
     noise_map
         The noise-map of the dataset.
     """
@@ -125,9 +125,9 @@ def chi_squared_map_complex_from(
     Chi_Squared = ((Residuals) / (Noise)) ** 2.0 = ((Data - Model)**2.0)/(Variances)
 
     Parameters
-    -----------
+    ----------
     residual_map
-        The residual-map of the model-simulator fit to the dataset.
+        The residual-map of the model-data fit to the dataset.
     noise_map
         The noise-map of the dataset.
     """
@@ -150,7 +150,7 @@ def chi_squared_complex_from(*, chi_squared_map: Structure) -> float:
     Parameters
     ----------
     chi_squared_map
-        The chi-squared-map of values of the model-simulator fit to the dataset.
+        The chi-squared-map of values of the model-data fit to the dataset.
     """
     chi_squared_real = float(np.sum(chi_squared_map.real))
     chi_squared_imag = float(np.sum(chi_squared_map.imag))
@@ -168,8 +168,8 @@ def noise_normalization_complex_from(*, noise_map: Structure) -> float:
     noise_map
         The masked noise-map of the dataset.
     """
-    noise_normalization_real = float(np.sum(np.log(2 * np.pi * noise_map.real ** 2.0)))
-    noise_normalization_imag = float(np.sum(np.log(2 * np.pi * noise_map.imag ** 2.0)))
+    noise_normalization_real = float(np.sum(np.log(2 * np.pi * noise_map.real**2.0)))
+    noise_normalization_imag = float(np.sum(np.log(2 * np.pi * noise_map.imag**2.0)))
     return noise_normalization_real + noise_normalization_imag
 
 
@@ -184,7 +184,7 @@ def residual_map_with_mask_from(
     The residual-map values in masked pixels are returned as zero.
 
     Parameters
-    -----------
+    ----------
     data
         The data that is fitted.
     mask
@@ -208,9 +208,9 @@ def normalized_residual_map_with_mask_from(
     The normalized residual-map values in masked pixels are returned as zero.
 
     Parameters
-    -----------
+    ----------
     residual_map
-        The residual-map of the model-simulator fit to the dataset.
+        The residual-map of the model-data fit to the dataset.
     noise_map
         The noise-map of the dataset.
     mask
@@ -235,9 +235,9 @@ def chi_squared_map_with_mask_from(
     The chi-squared-map values in masked pixels are returned as zero.
 
     Parameters
-    -----------
+    ----------
     residual_map
-        The residual-map of the model-simulator fit to the dataset.
+        The residual-map of the model-data fit to the dataset.
     noise_map
         The noise-map of the dataset.
     mask
@@ -263,13 +263,52 @@ def chi_squared_with_mask_from(*, chi_squared_map: Structure, mask: Mask) -> flo
     Parameters
     ----------
     chi_squared_map
-        The chi-squared-map of values of the model-simulator fit to the dataset.
+        The chi-squared-map of values of the model-data fit to the dataset.
     mask
         The mask applied to the chi-squared-map, where `False` entries are included in the calculation.
     """
     return float(np.sum(chi_squared_map[np.asarray(mask) == 0]))
 
 
+def chi_squared_with_mask_fast_from(
+    *, data: Structure, mask: Mask, model_data: Structure, noise_map: Structure
+) -> float:
+    """
+    Returns the chi-squared terms of each model data's fit to a masked dataset, by summing the masked
+    chi-squared-map of the fit.
+
+    The chi-squared values in masked pixels are omitted from the calculation.
+
+    Other methods in `fit_util` compute the `chi-squared` from the `residual_map` and `chi_squared_map`, which requires
+    that the `ndarrays` which are used to do this are created and stored in memory. For problems with large datasets
+    this can be computationally slow.
+
+    This function computes the `chi_squared` directly from the data, avoiding the need to store the data in memory
+    and offering faster tune times.
+
+    Parameters
+    ----------
+    chi_squared_map
+        The chi-squared-map of values of the model-data fit to the dataset.
+    mask
+        The mask applied to the chi-squared-map, where `False` entries are included in the calculation.
+    """
+
+    return float(
+        np.sum(
+            np.square(
+                np.divide(
+                    np.subtract(
+                        data,
+                        model_data,
+                    ),
+                    noise_map,
+                )
+            )[np.asarray(mask) == 0]
+        )
+    )
+
+
 def noise_normalization_with_mask_from(*, noise_map: Structure, mask: Mask) -> float:
     """
     Returns the noise-map normalization terms of masked noise-map, summing the noise_map value in every pixel as:
@@ -299,9 +338,9 @@ def normalized_residual_map_complex_with_mask_from(
     The normalized residual-map values in masked pixels are returned as zero.
 
     Parameters
-    -----------
+    ----------
     residual_map
-        The residual-map of the model-simulator fit to the dataset.
+        The residual-map of the model-data fit to the dataset.
     noise_map
         The noise-map of the dataset.
     mask
@@ -335,9 +374,9 @@ def chi_squared_map_complex_with_mask_from(
     The chi-squared-map values in masked pixels are returned as zero.
 
     Parameters
-    -----------
+    ----------
     residual_map
-        The residual-map of the model-simulator fit to the dataset.
+        The residual-map of the model-data fit to the dataset.
     noise_map
         The noise-map of the dataset.
     mask
@@ -375,7 +414,7 @@ def chi_squared_complex_with_mask_from(
     Parameters
     ----------
     chi_squared_map
-        The chi-squared-map of values of the model-simulator fit to the dataset.
+        The chi-squared-map of values of the model-data fit to the dataset.
     mask
         The mask applied to the chi-squared-map, where `False` entries are included in the calculation.
     """
@@ -384,6 +423,26 @@ def chi_squared_complex_with_mask_from(
     return chi_squared_real + chi_squared_imag
 
 
+def chi_squared_with_noise_covariance_from(
+    *, residual_map: Structure, noise_covariance_matrix_inv: np.ndarray
+) -> float:
+    """
+    Returns the chi-squared value of the fit of model-data to a masked dataset, where
+    the noise correlation is described by a covariance matrix of n^2 x n^2 dimensions.
+
+    Chi_Squared = r C^{-1} r, where C^{-1} is the inverse of the covariance matrix
+
+    Parameters
+    ----------
+    residual_map
+        The residual-map of the model-data fit to the dataset.
+    noise_covariance_matrix_inv
+        The inverse of the noise covariance matrix.
+    """
+
+    return residual_map @ noise_covariance_matrix_inv @ residual_map
+
+
 def noise_normalization_complex_with_mask_from(
     *, noise_map: Structure, mask: Mask
 ) -> float:
@@ -419,7 +478,7 @@ def log_likelihood_from(*, chi_squared: float, noise_normalization: float) -> fl
     Parameters
     ----------
     chi_squared
-        The chi-squared term for the model-simulator fit to the dataset.
+        The chi-squared term for the model-data fit to the dataset.
     noise_normalization
         The normalization noise_map-term for the dataset's noise-map.
     """
@@ -458,7 +517,7 @@ def log_evidence_from(
 ) -> float:
     """
     Returns the log evidence of an inversion's fit to a dataset, where the log evidence includes a number of terms which
-    quantify the complexity of an inversion's reconstruction (see the `LEq` module):
+    quantify the complexity of an inversion's reconstruction (see the `Inversion` module):
 
     Log Evidence = -0.5*[Chi_Squared_Term + Regularization_Term + Log(Covariance_Regularization_Term) -
                            Log(Regularization_Matrix_Term) + Noise_Term]
