@@ -2,13 +2,13 @@ import numpy as np
 from typing import Callable
 
 from autoarray.plot.abstract_plotters import Plotter
-from autoarray.plot.mat_wrap.visuals import Visuals1D
-from autoarray.plot.mat_wrap.visuals import Visuals2D
-from autoarray.plot.mat_wrap.include import Include1D
-from autoarray.plot.mat_wrap.include import Include2D
-from autoarray.plot.mat_wrap.mat_plot import MatPlot1D
-from autoarray.plot.mat_wrap.mat_plot import MatPlot2D
-from autoarray.plot.mat_wrap.mat_plot import AutoLabels
+from autoarray.plot.visuals.one_d import Visuals1D
+from autoarray.plot.visuals.two_d import Visuals2D
+from autoarray.plot.include.one_d import Include1D
+from autoarray.plot.include.two_d import Include2D
+from autoarray.plot.mat_plot.one_d import MatPlot1D
+from autoarray.plot.mat_plot.two_d import MatPlot2D
+from autoarray.plot.auto_labels import AutoLabels
 from autoarray.fit.fit_interferometer import FitInterferometer
 
 
@@ -23,6 +23,7 @@ class FitInterferometerPlotterMeta(Plotter):
         mat_plot_2d: MatPlot2D = MatPlot2D(),
         visuals_2d: Visuals2D = Visuals2D(),
         include_2d: Include2D = Include2D(),
+        residuals_symmetric_cmap: bool = True,
     ):
         """
         Plots the attributes of `FitInterferometer` objects using the matplotlib method `imshow()` and many
@@ -55,6 +56,9 @@ class FitInterferometerPlotterMeta(Plotter):
             Contains 2D visuals that can be overlaid on 2D plots.
         include_2d
             Specifies which attributes of the `FitInterferometer` are extracted and plotted as visuals for 2D plots.
+        residuals_symmetric_cmap
+            If true, the `residual_map` and `normalized_residual_map` are plotted with a symmetric color map such
+            that `abs(vmin) = abs(vmax)`.
         """
         super().__init__(
             mat_plot_1d=mat_plot_1d,
@@ -67,6 +71,7 @@ class FitInterferometerPlotterMeta(Plotter):
 
         self.fit = fit
         self.get_visuals_2d_real_space = get_visuals_2d_real_space
+        self.residuals_symmetric_cmap = residuals_symmetric_cmap
 
     def figures_2d(
         self,
@@ -91,43 +96,43 @@ class FitInterferometerPlotterMeta(Plotter):
         """
         Plots the individual attributes of the plotter's `FitInterferometer` object in 1D and 2D.
 
-        The API is such that every plottable attribute of the `Interferometer` object is an input parameter of type 
+        The API is such that every plottable attribute of the `Interferometer` object is an input parameter of type
         bool of the function, which if switched to `True` means that it is plotted.
 
         Parameters
         ----------
         visibilities
-            Whether or not to make a 2D plot (via `scatter`) of the visibility data.
+            Whether to make a 2D plot (via `scatter`) of the visibility data.
         noise_map
-            Whether or not to make a 2D plot (via `scatter`) of the noise-map.
+            Whether to make a 2D plot (via `scatter`) of the noise-map.
         signal_to_noise_map
-            Whether or not to make a 2D plot (via `scatter`) of the signal-to-noise-map.
+            Whether to make a 2D plot (via `scatter`) of the signal-to-noise-map.
         model_visibilities
-            Whether or not to make a 2D plot (via `scatter`) of the model visibility data.
+            Whether to make a 2D plot (via `scatter`) of the model visibility data.
         residual_map_real
-            Whether or not to make a 1D plot (via `plot`) of the real component of the residual map.
+            Whether to make a 1D plot (via `plot`) of the real component of the residual map.
         residual_map_imag
-            Whether or not to make a 1D plot (via `plot`) of the imaginary component of the residual map.
+            Whether to make a 1D plot (via `plot`) of the imaginary component of the residual map.
         normalized_residual_map_real
-            Whether or not to make a 1D plot (via `plot`) of the real component of the normalized residual map.
+            Whether to make a 1D plot (via `plot`) of the real component of the normalized residual map.
         normalized_residual_map_imag
-            Whether or not to make a 1D plot (via `plot`) of the imaginary component of the normalized residual map.
+            Whether to make a 1D plot (via `plot`) of the imaginary component of the normalized residual map.
         chi_squared_map_real
-            Whether or not to make a 1D plot (via `plot`) of the real component of the chi-squared map.
+            Whether to make a 1D plot (via `plot`) of the real component of the chi-squared map.
         chi_squared_map_imag
-            Whether or not to make a 1D plot (via `plot`) of the imaginary component of the chi-squared map.
+            Whether to make a 1D plot (via `plot`) of the imaginary component of the chi-squared map.
         dirty_image
-            Whether or not to make a 2D plot (via `imshow`) of the dirty image.
+            Whether to make a 2D plot (via `imshow`) of the dirty image.
         dirty_noise_map
-            Whether or not to make a 2D plot (via `imshow`) of the dirty noise map.
+            Whether to make a 2D plot (via `imshow`) of the dirty noise map.
         dirty_model_image
-            Whether or not to make a 2D plot (via `imshow`) of the dirty model image.
+            Whether to make a 2D plot (via `imshow`) of the dirty model image.
         dirty_residual_map
-            Whether or not to make a 2D plot (via `imshow`) of the dirty residual map.
+            Whether to make a 2D plot (via `imshow`) of the dirty residual map.
         dirty_normalized_residual_map
-            Whether or not to make a 2D plot (via `imshow`) of the dirty normalized residual map.
+            Whether to make a 2D plot (via `imshow`) of the dirty normalized residual map.
         dirty_chi_squared_map
-            Whether or not to make a 2D plot (via `imshow`) of the dirty chi-squared map.
+            Whether to make a 2D plot (via `imshow`) of the dirty chi-squared map.
         """
 
         if visibilities:
@@ -169,7 +174,7 @@ class FitInterferometerPlotterMeta(Plotter):
         if residual_map_real:
             self.mat_plot_1d.plot_yx(
                 y=np.real(self.fit.residual_map),
-                x=self.fit.interferometer.uv_distances / 10 ** 3.0,
+                x=self.fit.interferometer.uv_distances / 10**3.0,
                 visuals_1d=self.visuals_1d,
                 auto_labels=AutoLabels(
                     title="Residual Map vs UV-Distance (real)",
@@ -182,7 +187,7 @@ class FitInterferometerPlotterMeta(Plotter):
         if residual_map_imag:
             self.mat_plot_1d.plot_yx(
                 y=np.imag(self.fit.residual_map),
-                x=self.fit.interferometer.uv_distances / 10 ** 3.0,
+                x=self.fit.interferometer.uv_distances / 10**3.0,
                 visuals_1d=self.visuals_1d,
                 auto_labels=AutoLabels(
                     title="Residual Map vs UV-Distance (imag)",
@@ -197,7 +202,7 @@ class FitInterferometerPlotterMeta(Plotter):
 
             self.mat_plot_1d.plot_yx(
                 y=np.real(self.fit.residual_map),
-                x=self.fit.interferometer.uv_distances / 10 ** 3.0,
+                x=self.fit.interferometer.uv_distances / 10**3.0,
                 visuals_1d=self.visuals_1d,
                 auto_labels=AutoLabels(
                     title="Normalized Residual Map vs UV-Distance (real)",
@@ -210,7 +215,7 @@ class FitInterferometerPlotterMeta(Plotter):
         if normalized_residual_map_imag:
             self.mat_plot_1d.plot_yx(
                 y=np.imag(self.fit.residual_map),
-                x=self.fit.interferometer.uv_distances / 10 ** 3.0,
+                x=self.fit.interferometer.uv_distances / 10**3.0,
                 visuals_1d=self.visuals_1d,
                 auto_labels=AutoLabels(
                     title="Normalized Residual Map vs UV-Distance (imag)",
@@ -225,7 +230,7 @@ class FitInterferometerPlotterMeta(Plotter):
 
             self.mat_plot_1d.plot_yx(
                 y=np.real(self.fit.residual_map),
-                x=self.fit.interferometer.uv_distances / 10 ** 3.0,
+                x=self.fit.interferometer.uv_distances / 10**3.0,
                 visuals_1d=self.visuals_1d,
                 auto_labels=AutoLabels(
                     title="Chi-Squared Map vs UV-Distance (real)",
@@ -238,7 +243,7 @@ class FitInterferometerPlotterMeta(Plotter):
         if chi_squared_map_imag:
             self.mat_plot_1d.plot_yx(
                 y=np.imag(self.fit.residual_map),
-                x=self.fit.interferometer.uv_distances / 10 ** 3.0,
+                x=self.fit.interferometer.uv_distances / 10**3.0,
                 visuals_1d=self.visuals_1d,
                 auto_labels=AutoLabels(
                     title="Chi-Squared Map vs UV-Distance (imag)",
@@ -288,6 +293,12 @@ class FitInterferometerPlotterMeta(Plotter):
                 ),
             )
 
+        cmap_original = self.mat_plot_2d.cmap
+
+        if self.residuals_symmetric_cmap:
+
+            self.mat_plot_2d.cmap = self.mat_plot_2d.cmap.symmetric
+
         if dirty_residual_map:
 
             self.mat_plot_2d.plot_array(
@@ -309,6 +320,10 @@ class FitInterferometerPlotterMeta(Plotter):
                 ),
             )
 
+        if self.residuals_symmetric_cmap:
+
+            self.mat_plot_2d.cmap = cmap_original
+
         if dirty_chi_squared_map:
 
             self.mat_plot_2d.plot_array(
@@ -349,37 +364,37 @@ class FitInterferometerPlotterMeta(Plotter):
         Parameters
         ----------
         visibilities
-            Whether or not to make a 2D plot (via `scatter`) of the visibility data.
+            Whether to make a 2D plot (via `scatter`) of the visibility data.
         noise_map
-            Whether or not to make a 2D plot (via `scatter`) of the noise-map.
+            Whether to make a 2D plot (via `scatter`) of the noise-map.
         signal_to_noise_map
-            Whether or not to make a 2D plot (via `scatter`) of the signal-to-noise-map.
+            Whether to make a 2D plot (via `scatter`) of the signal-to-noise-map.
         model_visibilities
-            Whether or not to make a 2D plot (via `scatter`) of the model visibility data.
+            Whether to make a 2D plot (via `scatter`) of the model visibility data.
         residual_map_real
-            Whether or not to make a 1D plot (via `plot`) of the real component of the residual map.
+            Whether to make a 1D plot (via `plot`) of the real component of the residual map.
         residual_map_imag
-            Whether or not to make a 1D plot (via `plot`) of the imaginary component of the residual map.
+            Whether to make a 1D plot (via `plot`) of the imaginary component of the residual map.
         normalized_residual_map_real
-            Whether or not to make a 1D plot (via `plot`) of the real component of the normalized residual map.
+            Whether to make a 1D plot (via `plot`) of the real component of the normalized residual map.
         normalized_residual_map_imag
-            Whether or not to make a 1D plot (via `plot`) of the imaginary component of the normalized residual map.
+            Whether to make a 1D plot (via `plot`) of the imaginary component of the normalized residual map.
         chi_squared_map_real
-            Whether or not to make a 1D plot (via `plot`) of the real component of the chi-squared map.
+            Whether to make a 1D plot (via `plot`) of the real component of the chi-squared map.
         chi_squared_map_imag
-            Whether or not to make a 1D plot (via `plot`) of the imaginary component of the chi-squared map.
+            Whether to make a 1D plot (via `plot`) of the imaginary component of the chi-squared map.
         dirty_image
-            Whether or not to make a 2D plot (via `imshow`) of the dirty image.
+            Whether to make a 2D plot (via `imshow`) of the dirty image.
         dirty_noise_map
-            Whether or not to make a 2D plot (via `imshow`) of the dirty noise map.
+            Whether to make a 2D plot (via `imshow`) of the dirty noise map.
         dirty_model_image
-            Whether or not to make a 2D plot (via `imshow`) of the dirty model image.
+            Whether to make a 2D plot (via `imshow`) of the dirty model image.
         dirty_residual_map
-            Whether or not to make a 2D plot (via `imshow`) of the dirty residual map.
+            Whether to make a 2D plot (via `imshow`) of the dirty residual map.
         dirty_normalized_residual_map
-            Whether or not to make a 2D plot (via `imshow`) of the dirty normalized residual map.
+            Whether to make a 2D plot (via `imshow`) of the dirty normalized residual map.
         dirty_chi_squared_map
-            Whether or not to make a 2D plot (via `imshow`) of the dirty chi-squared map.
+            Whether to make a 2D plot (via `imshow`) of the dirty chi-squared map.
         auto_filename
             The default filename of the output subplot if written to hard-disk.
         """
