@@ -463,17 +463,20 @@ class AbstractInversion:
 
         S is the vector of reconstructed inversion values.
         """
-        if not self.settings.use_positive_only_solver:
-            return inversion_util.reconstruction_positive_negative_from(
+        if self.settings.use_positive_only_solver:
+
+            return inversion_util.reconstruction_positive_only_from(
                 data_vector=self.data_vector,
                 curvature_reg_matrix=self.curvature_reg_matrix_solver,
                 settings=self.settings,
             )
 
-        return inversion_util.reconstruction_positive_only_from(
+        mapper_param_range_list = self.param_range_list_from(cls=AbstractMapper)
+
+        return inversion_util.reconstruction_positive_negative_from(
             data_vector=self.data_vector,
             curvature_reg_matrix=self.curvature_reg_matrix_solver,
-            settings=self.settings,
+            mapper_param_range_list=mapper_param_range_list,
         )
 
     @cached_property
@@ -628,8 +631,8 @@ class AbstractInversion:
             return 2.0 * np.sum(
                 np.log(np.diag(np.linalg.cholesky(self.curvature_reg_matrix_reduced)))
             )
-        except np.linalg.LinAlgError:
-            raise exc.InversionException()
+        except np.linalg.LinAlgError as e:
+            raise exc.InversionException() from e
 
     @cached_property
     @profile_func
@@ -671,8 +674,8 @@ class AbstractInversion:
                         np.diag(np.linalg.cholesky(self.regularization_matrix_reduced))
                     )
                 )
-            except np.linalg.LinAlgError:
-                raise exc.InversionException()
+            except np.linalg.LinAlgError as e:
+                raise exc.InversionException() from e
 
     @property
     def errors_with_covariance(self) -> np.ndarray:
