@@ -1,8 +1,9 @@
 import numpy as np
-from scipy.linalg import cho_solve
-from scipy.optimize import lsq_linear, nnls
+from scipy.optimize import nnls
 from typing import List, Optional
 
+from autoconf import conf
+
 from autoarray.inversion.inversion.settings import SettingsInversion
 
 from autoarray import numba_util
@@ -258,7 +259,8 @@ def mapped_reconstructed_data_via_mapping_matrix_from(
 def reconstruction_positive_negative_from(
     data_vector: np.ndarray,
     curvature_reg_matrix: np.ndarray,
-    settings: SettingsInversion = SettingsInversion(),
+    mapper_param_range_list,
+    force_check_reconstruction: bool = False,
 ):
     """
     Solve the linear system [F + reg_coeff*H] S = D -> S = [F + reg_coeff*H]^-1 D given by equation (12)
@@ -271,8 +273,11 @@ def reconstruction_positive_negative_from(
     are nonphysical or undesirable.
 
     This function checks that the solution does not give a linear algebra error (e.g. because the input matrix is
-    not positive-definitive) and that it avoids solutions where all reconstructed values go to the same value. If these
-    occur it raises an exception.
+    not positive-definitive).
+
+    It also explicitly checks solutions where all reconstructed values go to the same value, and raises an exception if
+    this occurs. This solution occurs in many scenarios when it is clear not a valid solution, and therefore is checked
+    for and removed.
 
     Parameters
     ----------
@@ -280,9 +285,12 @@ def reconstruction_positive_negative_from(
         The `data_vector` D which is solved for.
     curvature_reg_matrix
         The sum of the curvature and regularization matrices.
-    settings
-        Controls the settings of the inversion, for this function where the solution is checked to not be all
-        the same values.
+    mapper_param_range_list
+        A list of lists, where each list contains the range of values in the solution vector (reconstruction) that
+        correspond to values that are part of a mapper's mesh.
+    force_check_reconstruction
+        If `True`, the reconstruction is forced to check for solutions where all reconstructed values go to the same
+        value irrespective of the configuration file value.
 
     Returns
     -------
@@ -294,6 +302,18 @@ def reconstruction_positive_negative_from(
     except np.linalg.LinAlgError as e:
         raise exc.InversionException() from e
 
+    if (
+        conf.instance["general"]["inversion"]["check_reconstruction"]
+        or force_check_reconstruction
+    ):
+
+        for mapper_param_range in mapper_param_range_list:
+            if np.allclose(
+                a=reconstruction[mapper_param_range[0] : mapper_param_range[1]],
+                b=reconstruction[mapper_param_range[0]],
+            ):
+                raise exc.InversionException()
+
     return reconstruction
 
 
