@@ -53,7 +53,7 @@ def w_tilde_data_interferometer_from(
 
     w_tilde_data = np.zeros(image_pixels)
 
-    weight_map_real = visibilities_real / noise_map_real ** 2.0
+    weight_map_real = visibilities_real / noise_map_real**2.0
 
     for ip0 in range(image_pixels):
 
@@ -125,27 +125,37 @@ def w_tilde_curvature_preload_interferometer_from(
     This calculation has a lot of redundancy, because it uses the (y,x) *spatial offset* between the image pixels. For
     example, if two image pixel are next to one another by the same spacing the same value will be computed via the
     NUFFT. For the example mask above:
+
     - The value precomputed for pixel pair [0,1] is the same as pixel pairs [1,2], [3,4], [4,5], [6,7] and [7,9].
+
     - The value precomputed for pixel pair [0,3] is the same as pixel pairs [1,4], [2,5], [3,6], [4,7] and [5,8].
+
     - The values of pixels paired with themselves are also computed repeatedly for the standard calculation (e.g. 9
-    times using the mask above).
+      times using the mask above).
+
     The `w_tilde_preload` method instead only computes each value once. To do this, it stores the preload values in a
     matrix of dimensions [shape_masked_pixels_y, shape_masked_pixels_x, 2], where `shape_masked_pixels` is the (y,x)
     size of the vertical and horizontal extent of unmasked pixels, e.g. the spatial extent over which the real space
     grid extends.
     Each entry in the matrix `w_tilde_preload[:,:,0]` provides the the precomputed NUFFT value mapping an image pixel
     to a pixel offset by that much in the y and x directions, for example:
+
     - w_tilde_preload[0,0,0] gives the precomputed values of image pixels that are offset in the y direction by 0 and
-    in the x direction by 0 - the values of pixels paired with themselves.
+      in the x direction by 0 - the values of pixels paired with themselves.
+
     - w_tilde_preload[1,0,0] gives the precomputed values of image pixels that are offset in the y direction by 1 and
-    in the x direction by 0 - the values of pixel pairs [0,3], [1,4], [2,5], [3,6], [4,7] and [5,8]
+      in the x direction by 0 - the values of pixel pairs [0,3], [1,4], [2,5], [3,6], [4,7] and [5,8]
+
     - w_tilde_preload[0,1,0] gives the precomputed values of image pixels that are offset in the y direction by 0 and
-    in the x direction by 1 - the values of pixel pairs [0,1], [1,2], [3,4], [4,5], [6,7] and [7,9].
+      in the x direction by 1 - the values of pixel pairs [0,1], [1,2], [3,4], [4,5], [6,7] and [7,9].
+
     Flipped pairs:
+
     The above preloaded values pair all image pixel NUFFT values when a pixel is to the right and / or down of the
     first image pixel. However, one must also precompute pairs where the paired pixel is to the left of the host
     pixels. These pairings are stored in `w_tilde_preload[:,:,1]`, and the ordering of these pairings is flipped in the
     x direction to make it straight forward to use this matrix when computing w_tilde.
+
     Parameters
     ----------
     noise_map_real
@@ -553,11 +563,11 @@ def curvature_matrix_via_w_tilde_curvature_preload_interferometer_from(
     This function speeds this calculation up in two ways:
 
     1) Instead of using `w_tilde` (dimensions [image_pixels, image_pixels] it uses `w_tilde_preload` (dimensions
-    [image_pixels, 2]). The massive reduction in the size of this matrix in memory allows for much fast computation.
+       [image_pixels, 2]). The massive reduction in the size of this matrix in memory allows for much fast computation.
 
     2) It omits the `mapping_matrix` and instead uses directly the 1D vector that maps every image pixel to a source
-    pixel `native_index_for_slim_index`. This exploits the sparsity in the `mapping_matrix` to directly
-    compute the `curvature_matrix` (e.g. it condenses the triple matrix multiplication into a double for loop!).
+       pixel `native_index_for_slim_index`. This exploits the sparsity in the `mapping_matrix` to directly
+       compute the `curvature_matrix` (e.g. it condenses the triple matrix multiplication into a double for loop!).
 
     Parameters
     ----------
@@ -565,16 +575,16 @@ def curvature_matrix_via_w_tilde_curvature_preload_interferometer_from(
         A matrix that precomputes the values for fast computation of w_tilde, which in this function is used to bypass
         the creation of w_tilde altogether and go directly to the `curvature_matrix`.
     pix_indexes_for_sub_slim_index
-        The mappings from a data sub-pixel index to a pixelization pixel index.
+        The mappings from a data sub-pixel index to a pixelization's mesh pixel index.
     pix_size_for_sub_slim_index
-        The number of mappings between each data sub pixel and pixelizaiton pixel.
+        The number of mappings between each data sub pixel and pixelization pixel.
     pix_weights_for_sub_slim_index
-        The weights of the mappings of every data sub pixel and pixelizaiton pixel.
+        The weights of the mappings of every data sub pixel and pixelization pixel.
     native_index_for_slim_index
         An array of shape [total_unmasked_pixels*sub_size] that maps every unmasked sub-pixel to its corresponding
         native 2D pixel using its (y,x) pixel indexes.
     pix_pixels
-        The total number of pixels in the pixelization that reconstructs the data.
+        The total number of pixels in the pixelization's mesh that reconstructs the data.
 
     Returns
     -------
@@ -668,11 +678,11 @@ def curvature_matrix_via_w_tilde_curvature_preload_interferometer_from_1(
     This function speeds this calculation up in two ways:
 
     1) Instead of using `w_tilde` (dimensions [image_pixels, image_pixels] it uses `w_tilde_preload` (dimensions
-    [image_pixels, 2]). The massive reduction in the size of this matrix in memory allows for much fast computation.
+       [image_pixels, 2]). The massive reduction in the size of this matrix in memory allows for much fast computation.
 
     2) It omits the `mapping_matrix` and instead uses directly the 1D vector that maps every image pixel to a source
-    pixel `native_index_for_slim_index`. This exploits the sparsity in the `mapping_matrix` to directly
-    compute the `curvature_matrix` (e.g. it condenses the triple matrix multiplication into a double for loop!).
+       pixel `native_index_for_slim_index`. This exploits the sparsity in the `mapping_matrix` to directly
+       compute the `curvature_matrix` (e.g. it condenses the triple matrix multiplication into a double for loop!).
 
     Parameters
     ----------
@@ -680,16 +690,16 @@ def curvature_matrix_via_w_tilde_curvature_preload_interferometer_from_1(
         A matrix that precomputes the values for fast computation of w_tilde, which in this function is used to bypass
         the creation of w_tilde altogether and go directly to the `curvature_matrix`.
     pix_indexes_for_sub_slim_index
-        The mappings from a data sub-pixel index to a pixelization pixel index.
+        The mappings from a data sub-pixel index to a pixelization's mesh pixel index.
     pix_size_for_sub_slim_index
-        The number of mappings between each data sub pixel and pixelizaiton pixel.
+        The number of mappings between each data sub pixel and pixelization pixel.
     pix_weights_for_sub_slim_index
-        The weights of the mappings of every data sub pixel and pixelizaiton pixel.
+        The weights of the mappings of every data sub pixel and pixelization pixel.
     native_index_for_slim_index
         An array of shape [total_unmasked_pixels*sub_size] that maps every unmasked sub-pixel to its corresponding
         native 2D pixel using its (y,x) pixel indexes.
     pix_pixels
-        The total number of pixels in the pixelization that reconstructs the data.
+        The total number of pixels in the pixelization's mesh that reconstructs the data.
 
     Returns
     -------
@@ -733,9 +743,6 @@ def curvature_matrix_via_w_tilde_curvature_preload_interferometer_from_1(
 @numba_util.jit()
 def curvature_matrix_via_w_tilde_curvature_preload_interferometer_from_2(
     curvature_preload: np.ndarray,
-    pix_indexes_for_sub_slim_index: np.ndarray,
-    pix_size_for_sub_slim_index: np.ndarray,
-    pix_weights_for_sub_slim_index: np.ndarray,
     native_index_for_slim_index: np.ndarray,
     pix_pixels: int,
     sub_slim_indexes_for_pix_index,
@@ -753,11 +760,11 @@ def curvature_matrix_via_w_tilde_curvature_preload_interferometer_from_2(
     This function speeds this calculation up in two ways:
 
     1) Instead of using `w_tilde` (dimensions [image_pixels, image_pixels] it uses `w_tilde_preload` (dimensions
-    [image_pixels, 2]). The massive reduction in the size of this matrix in memory allows for much fast computation.
+       [image_pixels, 2]). The massive reduction in the size of this matrix in memory allows for much fast computation.
 
     2) It omits the `mapping_matrix` and instead uses directly the 1D vector that maps every image pixel to a source
-    pixel `native_index_for_slim_index`. This exploits the sparsity in the `mapping_matrix` to directly
-    compute the `curvature_matrix` (e.g. it condenses the triple matrix multiplication into a double for loop!).
+       pixel `native_index_for_slim_index`. This exploits the sparsity in the `mapping_matrix` to directly
+       compute the `curvature_matrix` (e.g. it condenses the triple matrix multiplication into a double for loop!).
 
     Parameters
     ----------
@@ -765,16 +772,103 @@ def curvature_matrix_via_w_tilde_curvature_preload_interferometer_from_2(
         A matrix that precomputes the values for fast computation of w_tilde, which in this function is used to bypass
         the creation of w_tilde altogether and go directly to the `curvature_matrix`.
     pix_indexes_for_sub_slim_index
-        The mappings from a data sub-pixel index to a pixelization pixel index.
+        The mappings from a data sub-pixel index to a pixelization's mesh pixel index.
     pix_size_for_sub_slim_index
-        The number of mappings between each data sub pixel and pixelizaiton pixel.
+        The number of mappings between each data sub pixel and pixelization pixel.
     pix_weights_for_sub_slim_index
-        The weights of the mappings of every data sub pixel and pixelizaiton pixel.
+        The weights of the mappings of every data sub pixel and pixelization pixel.
     native_index_for_slim_index
         An array of shape [total_unmasked_pixels*sub_size] that maps every unmasked sub-pixel to its corresponding
         native 2D pixel using its (y,x) pixel indexes.
     pix_pixels
-        The total number of pixels in the pixelization that reconstructs the data.
+        The total number of pixels in the pixelization's mesh that reconstructs the data.
+
+    Returns
+    -------
+    ndarray
+        The curvature matrix `F` (see Warren & Dye 2003).
+    """
+
+    curvature_matrix = np.zeros((pix_pixels, pix_pixels))
+
+    for sp0 in range(pix_pixels):
+
+        ip_size_0 = sub_slim_sizes_for_pix_index[sp0]
+
+        for sp1 in range(sp0, pix_pixels):
+
+            val = 0.0
+            ip_size_1 = sub_slim_sizes_for_pix_index[sp1]
+
+            for ip0_tmp in range(ip_size_0):
+
+                ip0 = sub_slim_indexes_for_pix_index[sp0, ip0_tmp]
+                ip0_weight = sub_slim_weights_for_pix_index[sp0, ip0_tmp]
+
+                ip0_y, ip0_x = native_index_for_slim_index[ip0]
+
+                for ip1_tmp in range(ip_size_1):
+                    ip1 = sub_slim_indexes_for_pix_index[sp1, ip1_tmp]
+                    ip1_weight = sub_slim_weights_for_pix_index[sp1, ip1_tmp]
+
+                    ip1_y, ip1_x = native_index_for_slim_index[ip1]
+
+                    y_diff = ip1_y - ip0_y
+                    x_diff = ip1_x - ip0_x
+
+                    val += curvature_preload[y_diff, x_diff] * ip0_weight * ip1_weight
+
+            curvature_matrix[sp0, sp1] += val
+
+    for i in range(pix_pixels):
+        for j in range(i, pix_pixels):
+            curvature_matrix[j, i] = curvature_matrix[i, j]
+
+    return curvature_matrix
+
+
+@numba_util.jit()
+def curvature_matrix_via_w_tilde_curvature_preload_interferometer_from_2_no_interp(
+    curvature_preload: np.ndarray,
+    native_index_for_slim_index: np.ndarray,
+    pix_pixels: int,
+    sub_slim_indexes_for_pix_index,
+    sub_slim_sizes_for_pix_index,
+    sub_slim_weights_for_pix_index,
+) -> np.ndarray:
+    """
+    Returns the curvature matrix `F` (see Warren & Dye 2003) by computing it using `w_tilde_preload`
+    (see `w_tilde_preload_interferometer_from`) for an interferometer inversion.
+
+    To compute the curvature matrix via w_tilde the following matrix multiplication is normally performed:
+
+    curvature_matrix = mapping_matrix.T * w_tilde * mapping matrix
+
+    This function speeds this calculation up in two ways:
+
+    1) Instead of using `w_tilde` (dimensions [image_pixels, image_pixels] it uses `w_tilde_preload` (dimensions
+       [image_pixels, 2]). The massive reduction in the size of this matrix in memory allows for much fast computation.
+
+    2) It omits the `mapping_matrix` and instead uses directly the 1D vector that maps every image pixel to a source
+       pixel `native_index_for_slim_index`. This exploits the sparsity in the `mapping_matrix` to directly
+       compute the `curvature_matrix` (e.g. it condenses the triple matrix multiplication into a double for loop!).
+
+    Parameters
+    ----------
+    curvature_preload
+        A matrix that precomputes the values for fast computation of w_tilde, which in this function is used to bypass
+        the creation of w_tilde altogether and go directly to the `curvature_matrix`.
+    pix_indexes_for_sub_slim_index
+        The mappings from a data sub-pixel index to a pixelization's mesh pixel index.
+    pix_size_for_sub_slim_index
+        The number of mappings between each data sub pixel and pixelization pixel.
+    pix_weights_for_sub_slim_index
+        The weights of the mappings of every data sub pixel and pixelization pixel.
+    native_index_for_slim_index
+        An array of shape [total_unmasked_pixels*sub_size] that maps every unmasked sub-pixel to its corresponding
+        native 2D pixel using its (y,x) pixel indexes.
+    pix_pixels
+        The total number of pixels in the pixelization's mesh that reconstructs the data.
 
     Returns
     -------
