@@ -8,11 +8,9 @@ class MockRegularization(AbstractRegularization):
 
         self.regularization_matrix = regularization_matrix
 
-    def regularization_matrix_via_pixel_neighbors_from(
-        self, pixel_neighbors, pixel_neighbors_sizes
-    ):
+    def regularization_matrix_via_neighbors_from(self, neighbors, neighbors_sizes):
         return self.regularization_matrix
 
-    def regularization_matrix_from(self, mapper):
+    def regularization_matrix_from(self, linear_obj):
 
         return self.regularization_matrix
