@@ -1,12 +1,13 @@
 import numpy as np
 
 from autoconf import conf
+
+from autoarray.inversion.pixelization.mappers.abstract import AbstractMapper
 from autoarray.plot.abstract_plotters import Plotter
-from autoarray.plot.mat_wrap.visuals import Visuals2D
-from autoarray.plot.mat_wrap.include import Include2D
-from autoarray.plot.mat_wrap.mat_plot import MatPlot2D
-from autoarray.plot.mat_wrap.mat_plot import AutoLabels
-from autoarray.structures.arrays.uniform_2d import Array2D
+from autoarray.plot.visuals.two_d import Visuals2D
+from autoarray.plot.include.two_d import Include2D
+from autoarray.plot.mat_plot.two_d import MatPlot2D
+from autoarray.plot.auto_labels import AutoLabels
 from autoarray.inversion.inversion.abstract import AbstractInversion
 from autoarray.inversion.plot.mapper_plotters import MapperPlotter
 
@@ -18,6 +19,7 @@ class InversionPlotter(Plotter):
         mat_plot_2d: MatPlot2D = MatPlot2D(),
         visuals_2d: Visuals2D = Visuals2D(),
         include_2d: Include2D = Include2D(),
+        residuals_symmetric_cmap: bool = True,
     ):
         """
         Plots the attributes of `Inversion` objects using the matplotlib method `imshow()` and many other matplotlib
@@ -41,16 +43,20 @@ class InversionPlotter(Plotter):
             Contains 2D visuals that can be overlaid on 2D plots.
         include_2d
             Specifies which attributes of the `Inversion` are extracted and plotted as visuals for 2D plots.
+        residuals_symmetric_cmap
+            If true, the `residual_map` and `normalized_residual_map` are plotted with a symmetric color map such
+            that `abs(vmin) = abs(vmax)`.
         """
         super().__init__(
             mat_plot_2d=mat_plot_2d, include_2d=include_2d, visuals_2d=visuals_2d
         )
 
         self.inversion = inversion
+        self.residuals_symmetric_cmap = residuals_symmetric_cmap
 
     def get_visuals_2d_for_data(self) -> Visuals2D:
         return self.get_2d.via_mapper_for_data_from(
-            mapper=self.inversion.mapper_list[0]
+            mapper=self.inversion.cls_list_from(cls=AbstractMapper)[0]
         )
 
     def mapper_plotter_from(self, mapper_index: int) -> MapperPlotter:
@@ -69,7 +75,7 @@ class InversionPlotter(Plotter):
             An object that plots mappers which is used for plotting attributes of the inversion.
         """
         return MapperPlotter(
-            mapper=self.inversion.mapper_list[mapper_index],
+            mapper=self.inversion.cls_list_from(cls=AbstractMapper)[mapper_index],
             mat_plot_2d=self.mat_plot_2d,
             visuals_2d=self.visuals_2d,
             include_2d=self.include_2d,
@@ -85,7 +91,7 @@ class InversionPlotter(Plotter):
         Parameters
         ----------
         reconstructed_image
-            Whether or not to make a 2D plot (via `imshow`) of the reconstructed image data.
+            Whether to make a 2D plot (via `imshow`) of the reconstructed image data.
         """
         if reconstructed_image:
 
@@ -97,9 +103,9 @@ class InversionPlotter(Plotter):
                 ),
             )
 
-    def figures_2d_of_mapper(
+    def figures_2d_of_pixelization(
         self,
-        mapper_index: int = 0,
+        pixelization_index: int = 0,
         reconstructed_image: bool = False,
         reconstruction: bool = False,
         errors: bool = False,
@@ -116,30 +122,30 @@ class InversionPlotter(Plotter):
 
         Parameters
         ----------
-        mapper_index
+        pixelization_index
             The index of the `Mapper` in the `Inversion`'s `linear_obj_list` that is plotted.
         reconstructed_image
-            Whether or not to make a 2D plot (via `imshow`) of the mapper's reconstructed image data.
+            Whether to make a 2D plot (via `imshow`) of the mapper's reconstructed image data.
         reconstruction
-            Whether or not to make a 2D plot (via `imshow` or `fill`) of the mapper's source-plane reconstruction.
+            Whether to make a 2D plot (via `imshow` or `fill`) of the mapper's source-plane reconstruction.
         errors
-            Whether or not to make a 2D plot (via `imshow` or `fill`) of the mapper's source-plane errors.
+            Whether to make a 2D plot (via `imshow` or `fill`) of the mapper's source-plane errors.
         residual_map
-            Whether or not to make a 2D plot (via `imshow` or `fill`) of the mapper's source-plane residual map.
+            Whether to make a 2D plot (via `imshow` or `fill`) of the mapper's source-plane residual map.
         normalized_residual_map
-            Whether or not to make a 2D plot (via `imshow` or `fill`) of the mapper's source-plane normalized residual
+            Whether to make a 2D plot (via `imshow` or `fill`) of the mapper's source-plane normalized residual
             map.
         chi_squared_map
-            Whether or not to make a 2D plot (via `imshow` or `fill`) of the mapper's source-plane chi-squared map.
+            Whether to make a 2D plot (via `imshow` or `fill`) of the mapper's source-plane chi-squared map.
         residual_map
-            Whether or not to make a 2D plot (via `imshow` or `fill`) of the mapper's source-plane regularization
+            Whether to make a 2D plot (via `imshow` or `fill`) of the mapper's source-plane regularization
             weights.
         """
 
-        if not self.inversion.has_mapper:
+        if not self.inversion.has(cls=AbstractMapper):
             return
 
-        mapper_plotter = self.mapper_plotter_from(mapper_index=mapper_index)
+        mapper_plotter = self.mapper_plotter_from(mapper_index=pixelization_index)
 
         if reconstructed_image:
 
@@ -166,9 +172,10 @@ class InversionPlotter(Plotter):
                         "inversion"
                     ]["reconstruction_vmax_factor"]
 
-                    self.mat_plot_2d.cmap.kwargs["vmax"] = (
-                        reconstruction_vmax_factor
-                        * np.max(self.inversion.reconstruction)
+                    self.mat_plot_2d.cmap.kwargs[
+                        "vmax"
+                    ] = reconstruction_vmax_factor * np.max(
+                        self.inversion.reconstruction
                     )
                     vmax_custom = True
 
@@ -191,7 +198,7 @@ class InversionPlotter(Plotter):
             try:
 
                 mapper_plotter.plot_source_from(
-                    source_pixelization_values=self.inversion.error_dict[
+                    source_pixelization_values=self.inversion.errors_dict[
                         mapper_plotter.mapper
                     ],
                     auto_labels=AutoLabels(title="Errors", filename="errors"),
@@ -201,6 +208,12 @@ class InversionPlotter(Plotter):
 
                 pass
 
+        cmap_original = self.mat_plot_2d.cmap
+
+        if self.residuals_symmetric_cmap:
+
+            self.mat_plot_2d.cmap = self.mat_plot_2d.cmap.symmetric
+
         if residual_map:
 
             mapper_plotter.plot_source_from(
@@ -221,6 +234,8 @@ class InversionPlotter(Plotter):
                 ),
             )
 
+        self.mat_plot_2d.cmap = cmap_original
+
         if chi_squared_map:
 
             mapper_plotter.plot_source_from(
@@ -259,14 +274,22 @@ class InversionPlotter(Plotter):
         """
         self.open_subplot_figure(number_subplots=6)
 
-        self.figures_2d_of_mapper(mapper_index=mapper_index, reconstructed_image=True)
-        self.figures_2d_of_mapper(mapper_index=mapper_index, reconstruction=True)
-        self.figures_2d_of_mapper(mapper_index=mapper_index, errors=True)
-        self.figures_2d_of_mapper(mapper_index=mapper_index, residual_map=True)
-        self.figures_2d_of_mapper(
-            mapper_index=mapper_index, normalized_residual_map=True
+        self.figures_2d_of_pixelization(
+            pixelization_index=mapper_index, reconstructed_image=True
+        )
+        self.figures_2d_of_pixelization(
+            pixelization_index=mapper_index, reconstruction=True
+        )
+        self.figures_2d_of_pixelization(pixelization_index=mapper_index, errors=True)
+        self.figures_2d_of_pixelization(
+            pixelization_index=mapper_index, residual_map=True
+        )
+        self.figures_2d_of_pixelization(
+            pixelization_index=mapper_index, normalized_residual_map=True
+        )
+        self.figures_2d_of_pixelization(
+            pixelization_index=mapper_index, chi_squared_map=True
         )
-        self.figures_2d_of_mapper(mapper_index=mapper_index, chi_squared_map=True)
 
         self.mat_plot_2d.output.subplot_to_figure(
             auto_filename=f"{auto_filename}_{mapper_index}"
