@@ -2,13 +2,15 @@ import numpy as np
 from typing import Union
 
 from autoarray.plot.abstract_plotters import Plotter
-from autoarray.plot.mat_wrap.visuals import Visuals2D
-from autoarray.plot.mat_wrap.include import Include2D
-from autoarray.plot.mat_wrap.mat_plot import MatPlot2D
-from autoarray.plot.mat_wrap.mat_plot import AutoLabels
+from autoarray.plot.visuals.two_d import Visuals2D
+from autoarray.plot.include.two_d import Include2D
+from autoarray.plot.mat_plot.two_d import MatPlot2D
+from autoarray.plot.auto_labels import AutoLabels
 from autoarray.structures.arrays.uniform_2d import Array2D
-from autoarray.inversion.mappers.rectangular import MapperRectangularNoInterp
-from autoarray.inversion.mappers.voronoi import MapperVoronoiNoInterp
+from autoarray.inversion.pixelization.mappers.rectangular import (
+    MapperRectangularNoInterp,
+)
+from autoarray.inversion.pixelization.mappers.voronoi import MapperVoronoiNoInterp
 
 
 class MapperPlotter(Plotter):
@@ -62,6 +64,10 @@ class MapperPlotter(Plotter):
 
         Parameters
         ----------
+        interpolate_to_uniform
+            By default, the mesh's reconstruction is interpolated to a uniform 2D array for plotting. If the
+            reconstruction can be plotted in an alternative format (e.g. using Voronoi pixels for a Voronoi mesh)
+            settings `interpolate_to_uniform=False` plots the reconstruction using this.
         solution_vector
             A vector of values which can culor the pixels of the mapper's source pixels.
         """
@@ -69,8 +75,10 @@ class MapperPlotter(Plotter):
             mapper=self.mapper,
             visuals_2d=self.get_2d.via_mapper_for_source_from(mapper=self.mapper),
             interpolate_to_uniform=interpolate_to_uniform,
-            source_pixelilzation_values=solution_vector,
-            auto_labels=AutoLabels(title="Mapper", filename="mapper"),
+            source_pixelization_values=solution_vector,
+            auto_labels=AutoLabels(
+                title="Pixelization Mesh (Image-Plane)", filename="mapper"
+            ),
         )
 
     def subplot_image_and_mapper(
@@ -85,6 +93,10 @@ class MapperPlotter(Plotter):
 
         Parameters
         ----------
+        interpolate_to_uniform
+            By default, the mesh's reconstruction is interpolated to a uniform 2D array for plotting. If the
+            reconstruction can be plotted in an alternative format (e.g. using Voronoi pixels for a Voronoi mesh)
+            settings `interpolate_to_uniform=False` plots the reconstruction using this.
         image
             The image which is plotted on the subplot.
         """
@@ -93,7 +105,7 @@ class MapperPlotter(Plotter):
         self.mat_plot_2d.plot_array(
             array=image,
             visuals_2d=self.get_visuals_2d_for_data(),
-            auto_labels=AutoLabels(title="Image"),
+            auto_labels=AutoLabels(title="Image (Image-Plane)"),
         )
 
         if self.visuals_2d.pix_indexes is not None:
@@ -103,7 +115,8 @@ class MapperPlotter(Plotter):
             )
 
             self.mat_plot_2d.index_scatter.scatter_grid_indexes(
-                grid=self.mapper.source_grid_slim.mask.masked_grid, indexes=indexes
+                grid=self.mapper.source_plane_data_grid.mask.derive_grid.unmasked,
+                indexes=indexes,
             )
 
         self.figure_2d(interpolate_to_uniform=interpolate_to_uniform)
@@ -132,5 +145,5 @@ class MapperPlotter(Plotter):
             mapper=self.mapper,
             visuals_2d=self.get_visuals_2d_for_source(),
             auto_labels=auto_labels,
-            source_pixelilzation_values=source_pixelization_values,
+            source_pixelization_values=source_pixelization_values,
         )
