@@ -5,3 +5,5 @@ from .constant_zeroth import ConstantZeroth
 from .constant_split import ConstantSplit
 from .adaptive_brightness import AdaptiveBrightness
 from .adaptive_brightness_split import AdaptiveBrightnessSplit
+from .brightness_zeroth import BrightnessZeroth
+from .adaptive_brightness_split_zeroth import AdaptiveBrightnessSplitZeroth
