@@ -1,4 +1,9 @@
+from __future__ import annotations
 import numpy as np
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from autoarray.inversion.linear_obj.linear_obj import LinearObj
 
 try:
     import pylops
@@ -10,7 +15,8 @@ except ModuleNotFoundError:
 
 class AbstractRegularization:
     def __init__(self):
-        """ Abstract base class for a regularization-scheme, which is applied to a pixelization to enforce a \
+        """
+        Abstract base class for a regularization-scheme, which is applied to a pixelization to enforce a \
         smooth-source solution and prevent over-fitting noise_map in the hyper_galaxies. This is achieved by computing a \
         'regularization term' - which is the sum of differences in reconstructed flux between every set of neighboring \
         pixels. This regularization term is added to the solution's chi-squared as a penalty term. This effects \
@@ -25,7 +31,7 @@ class AbstractRegularization:
            coefficient is a 'hyper_galaxies-parameter' which determines how strongly we smooth the pixelization's reconstruction.
 
         The value of the coefficients(s) is set using the Bayesian framework of (Suyu 2006) and this \
-        is described further in the (*inversion.linear_eqn* class).
+        is described further in the (*inversion.inversion* class).
 
         The regularization matrix, H, is calculated by defining a set of B matrices which describe how the \
         pixels neighbor one another. For example, lets take a 3x3 square grid:
@@ -81,7 +87,7 @@ class AbstractRegularization:
         Whilst the example above used a square-grid with regularization to the right and downwards, this matrix \
         formalism can be extended to describe regularization in more directions (e.g. upwards, to the left).
 
-        It can also describe irpixelizations, e.g. an irVoronoi pixelization, where a B matrix is \
+        It can also describe irpixelizations, e.g. an irVoronoi mesh, where a B matrix is \
         computed for every shared Voronoi vertex of each Voronoi pixel. The number of B matrices is now equal to the \
         number of Voronoi vertices in the pixel with the most Voronoi vertices. However, we describe below a scheme to \
         compute this solution more efficiently.
@@ -110,7 +116,7 @@ class AbstractRegularization:
         Infact, going through the rigmarole of computing and multiplying B matrices like this is uncessary. It is \
         more computationally efficiently to directly compute H. This is possible, provided you know know all of the \
         neighboring pixel pairs (which, by definition, you need to know to set up the B matrices anyway). Thus, the \
-       'regularization_matrix_via_pixel_neighbors_from' functions in this module directly compute H from the pixel \
+       'regularization_matrix_via_neighbors_from' functions in this module directly compute H from the pixel \
         neighbors.
 
         # POSITIVE DEFINITE MATRIX #
@@ -119,7 +125,7 @@ class AbstractRegularization:
         use its determinant in the calculation.
 
         Parameters
-        -----------
+        ----------
         shape
             The dimensions of the rectangular grid of pixels (x_pixels, y_pixel)
         coefficients : (float,)
@@ -130,10 +136,43 @@ class AbstractRegularization:
     def __eq__(self, other):
         return self.__dict__ == other.__dict__ and self.__class__ is other.__class__
 
-    def regularization_weights_from(self, mapper):
+    def __hash__(self):
+        return id(self)
+
+    def regularization_weights_from(self, linear_obj: LinearObj) -> np.ndarray:
+        """
+        Returns the regularization weights of this regularization scheme.
+
+        The regularization weights define the level of regularization applied to each parameter in the linear object
+        (e.g. the ``pixels`` in a ``Mapper``).
+
+        For standard regularization (e.g. ``Constant``) are weights are equal, however for adaptive schemes
+        (e.g. ``AdaptiveBrightness``) they vary to adapt to the data being reconstructed.
+
+        Parameters
+        ----------
+        linear_obj
+            The linear object (e.g. a ``Mapper``) which uses these weights when performing regularization.
+
+        Returns
+        -------
+        The regularization weights.
+        """
         raise NotImplementedError
 
-    def regularization_matrix_from(self, mapper):
+    def regularization_matrix_from(self, linear_obj: LinearObj) -> np.ndarray:
+        """
+        Returns the regularization matrix of this regularization scheme.
+
+        Parameters
+        ----------
+        linear_obj
+            The linear object (e.g. a ``Mapper``) which uses this matrix to perform regularization.
+
+        Returns
+        -------
+        The regularization matrix.
+        """
         raise NotImplementedError
 
 
