@@ -31,8 +31,8 @@ class Layout1D:
         self.overscan = overscan
 
     def extract_overscan_array_1d_from(self, array):
-        return Array1D.manual_native(
-            array=array.native[self.overscan.slice],
+        return Array1D.no_mask(
+            values=array.native[self.overscan.slice],
             header=array.header,
             pixel_scales=array.pixel_scales,
         )
@@ -55,7 +55,7 @@ class Layout2D:
         project **PyAutoCTI** where rectangles of injected charge are contained on the image.
 
         Parameters
-        -----------
+        ----------
         shape_2d
             The two dimensional shape of the charge injection imaging, corresponding to the number of rows (pixels
             in parallel direction) and columns (pixels in serial direction).
@@ -216,8 +216,8 @@ class Layout2D:
                <---------S----------
         """
 
-        return Array2D.manual(
-            array=array.native[self.parallel_overscan.slice],
+        return Array2D.no_mask(
+            values=array.native[self.parallel_overscan.slice],
             header=array.header,
             pixel_scales=array.pixel_scales,
         )
@@ -270,8 +270,8 @@ class Layout2D:
                <---------S----------
         """
 
-        return Array2D.manual(
-            array=array.native[self.serial_overscan.slice],
+        return Array2D.no_mask(
+            values=array.native[self.serial_overscan.slice],
             header=array.header,
             pixel_scales=array.pixel_scales,
         )
