@@ -1,5 +1,4 @@
 import numpy as np
-from skimage.transform import rescale
 from typing import Tuple, Union
 import warnings
 
@@ -105,7 +104,7 @@ def total_sub_pixels_2d_from(mask_2d: np.ndarray, sub_size: int) -> int:
 
     total_sub_pixels = total_sub_pixels_from(mask=mask, sub_size=2)
     """
-    return total_pixels_2d_from(mask_2d) * sub_size ** 2
+    return total_pixels_2d_from(mask_2d) * sub_size**2
 
 
 @numba_util.jit()
@@ -117,7 +116,7 @@ def total_sparse_pixels_2d_from(
     by the pixelization grid.
 
     Parameters
-    -----------
+    ----------
     mask_2d
         The mask within which pixelization pixels must be inside
     unmasked_sparse_grid_pixel_centres
@@ -185,7 +184,7 @@ def mask_2d_circular_from(
             y_scaled = (y - centres_scaled[0]) * pixel_scales[0]
             x_scaled = (x - centres_scaled[1]) * pixel_scales[1]
 
-            r_scaled = np.sqrt(x_scaled ** 2 + y_scaled ** 2)
+            r_scaled = np.sqrt(x_scaled**2 + y_scaled**2)
 
             if r_scaled <= radius:
                 mask_2d[y, x] = False
@@ -242,7 +241,7 @@ def mask_2d_circular_annular_from(
             y_scaled = (y - centres_scaled[0]) * pixel_scales[0]
             x_scaled = (x - centres_scaled[1]) * pixel_scales[1]
 
-            r_scaled = np.sqrt(x_scaled ** 2 + y_scaled ** 2)
+            r_scaled = np.sqrt(x_scaled**2 + y_scaled**2)
 
             if outer_radius >= r_scaled >= inner_radius:
                 mask_2d[y, x] = False
@@ -305,7 +304,7 @@ def mask_2d_circular_anti_annular_from(
             y_scaled = (y - centres_scaled[0]) * pixel_scales[0]
             x_scaled = (x - centres_scaled[1]) * pixel_scales[1]
 
-            r_scaled = np.sqrt(x_scaled ** 2 + y_scaled ** 2)
+            r_scaled = np.sqrt(x_scaled**2 + y_scaled**2)
 
             if (
                 inner_radius >= r_scaled
@@ -375,7 +374,7 @@ def elliptical_radius_from(
     float
         The radius of the input scaled (y,x) coordinate on the ellipse's ellipitcal coordinate system.
     """
-    r_scaled = np.sqrt(x_scaled ** 2 + y_scaled ** 2)
+    r_scaled = np.sqrt(x_scaled**2 + y_scaled**2)
 
     theta_rotated = np.arctan2(y_scaled, x_scaled) + np.radians(angle)
 
@@ -383,7 +382,7 @@ def elliptical_radius_from(
     x_scaled_elliptical = r_scaled * np.cos(theta_rotated)
 
     return np.sqrt(
-        x_scaled_elliptical ** 2.0 + (y_scaled_elliptical / axis_ratio) ** 2.0
+        x_scaled_elliptical**2.0 + (y_scaled_elliptical / axis_ratio) ** 2.0
     )
 
 
@@ -426,7 +425,7 @@ def mask_2d_elliptical_from(
     Examples
     --------
     mask = mask_elliptical_from(
-        shape=(10, 10), pixel_scales=0.1, major_axis_radius=0.5, elliptical_comps=(0.333333, 0.0), centre=(0.0, 0.0))
+        shape=(10, 10), pixel_scales=0.1, major_axis_radius=0.5, ell_comps=(0.333333, 0.0), centre=(0.0, 0.0))
     """
 
     mask_2d = np.full(shape_native, True)
@@ -549,7 +548,7 @@ def blurring_mask_2d_from(
     should pad the input mask (and associated images).
 
     Parameters
-    -----------
+    ----------
     mask_2d
         A 2D array of bools, where `False` values are unmasked.
     kernel_shape_native
@@ -567,7 +566,7 @@ def blurring_mask_2d_from(
                      [True, False, True]
                      [True, True, True]])
 
-    blurring_mask = blurring_mask_from(mask=mask)
+    blurring_mask = blurring_from(mask=mask)
 
     """
 
@@ -604,11 +603,11 @@ def mask_2d_via_shape_native_and_native_for_slim(
     shape_native: Tuple[int, int], native_for_slim: np.ndarray
 ) -> np.ndarray:
     """
-    For a slimmed set of data that was computed by mapping unmasked values from a native 2D array of shape 
-    (total_y_pixels, total_x_pixels), map its slimmed indexes back to the original 2D array to create the 
+    For a slimmed set of data that was computed by mapping unmasked values from a native 2D array of shape
+    (total_y_pixels, total_x_pixels), map its slimmed indexes back to the original 2D array to create the
     native 2D mask.
 
-    This uses an array 'native_for_slim' of shape [total_masked_pixels[ where each index gives the native 2D pixel 
+    This uses an array 'native_for_slim' of shape [total_masked_pixels[ where each index gives the native 2D pixel
     indexes of the slimmed array's unmasked pixels, for example:
 
     - If native_for_slim[0] = [0,0], the first value of the slimmed array maps to the pixel [0,0] of the native 2D array.
@@ -642,6 +641,74 @@ def mask_2d_via_shape_native_and_native_for_slim(
     return mask
 
 
+@numba_util.jit()
+def mask_slim_indexes_from(
+    mask_2d: np.ndarray, return_masked_indexes: bool = True
+) -> np.ndarray:
+    """
+    Returns a 1D array listing all masked (`value=True`) or unmasked pixel indexes (`value=False`) in the mask.
+
+    For example, for the following ``Mask2D`` for ``sub_size=1``:
+
+    ::
+        [[True,  True,  True, True]
+         [True, False, False, True],
+         [True, False,  True, True],
+         [True,  True,  True, True]]
+
+    This has three unmasked (``False`` values) which have the ``slim`` indexes, there ``unmasked_slim`` is:
+
+    ::
+        [0, 1, 2]
+
+    For a ``Mask2D`` with ``sub_size=2`` each unmasked ``False`` entry is split into a sub-pixel of size 2x2.
+    Therefore the array ``unmasked_slim`` becomes:
+
+    ::
+        [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]
+
+    The ``slim`` indexes are by definition the unmasked pixels, therefore this will always return an array of
+    ascending integers with size the total number of unmasked pixels multiplied by the ``sub_size**2``.
+
+    Parameters
+    ----------
+    mask_2d
+        The mask for which the 1D unmasked pixel indexes are computed.
+    return_masked_indexes
+        Whether to return the masked index values (`value=True`) or the unmasked index values (`value=False`).
+
+    Returns
+    -------
+    np.ndarray
+        The 1D indexes of all unmasked pixels on the mask.
+    """
+
+    mask_pixel_total = 0
+
+    for y in range(0, mask_2d.shape[0]):
+        for x in range(0, mask_2d.shape[1]):
+
+            if mask_2d[y, x] == return_masked_indexes:
+
+                mask_pixel_total += 1
+
+    mask_pixels = np.zeros(mask_pixel_total)
+    mask_index = 0
+    regular_index = 0
+
+    for y in range(0, mask_2d.shape[0]):
+        for x in range(0, mask_2d.shape[1]):
+
+            if mask_2d[y, x] == return_masked_indexes:
+
+                mask_pixels[mask_index] = regular_index
+                mask_index += 1
+
+            regular_index += 1
+
+    return mask_pixels
+
+
 @numba_util.jit()
 def check_if_edge_pixel(mask_2d: np.ndarray, y: int, x: int) -> bool:
     """
@@ -718,6 +785,24 @@ def edge_1d_indexes_from(mask_2d: np.ndarray) -> np.ndarray:
     An edge pixel is defined as a pixel on the mask which is unmasked (has a `False`) value and at least 1 of its 8
     direct neighbors is masked (is `True`).
 
+    For example, for the following ``Mask2D``:
+
+    ::
+        [[True,  True,  True,  True, True],
+         [True, False, False, False, True],
+         [True, False, False, False, True],
+         [True, False, False, False, True],
+         [True,  True,  True,  True, True]]
+
+    The `edge_slim` indexes (given via ``mask_2d.derive_indexes.edge_slim``) is given by:
+
+    ::
+         [0, 1, 2, 3, 5, 6, 7, 8]
+
+    Note that index 4 is skipped, which corresponds to the ``False`` value in the centre of the mask, because it
+    does not neighbor a ``True`` value in any one of the eight neighboring directions and is therefore not at
+    an edge.
+
     Parameters
     ----------
     mask_2d
@@ -857,6 +942,27 @@ def border_slim_indexes_from(mask_2d: np.ndarray) -> np.ndarray:
     The borders pixels are thus pixels which are on the exterior edge of the mask. For example, the inner ring of edge
     pixels in an annular mask are edge pixels but not borders pixels.
 
+    For example, for the following ``Mask2D``:
+
+    ::
+        [[True,  True,  True,  True,  True,  True,  True,  True, True],
+         [True, False, False, False, False, False, False, False, True],
+         [True, False,  True,  True,  True,  True,  True, False, True],
+         [True, False,  True, False, False, False,  True, False, True],
+         [True, False,  True, False,  True, False,  True, False, True],
+         [True, False,  True, False, False, False,  True, False, True],
+         [True, False,  True,  True,  True,  True,  True, False, True],
+         [True, False, False, False, False, False, False, False, True],
+         [True,  True,  True,  True,  True,  True,  True,  True, True]]
+
+    The `border_slim` indexes (given via ``mask_2d.derive_indexes.border_slim``) is given by:
+
+    ::
+         [0, 1, 2, 3, 5, 6, 7, 11, 12, 15, 16, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29]
+
+    The interior 8 ``False`` values are omitted, because although they are edge pixels (neighbor a ``True``) they
+    are not on the extreme exterior edge.
+
     Parameters
     ----------
     mask_2d
@@ -1020,6 +1126,8 @@ def rescaled_mask_2d_from(mask_2d: np.ndarray, rescale_factor: float) -> np.ndar
         The rescaled mask.
     """
 
+    from skimage.transform import rescale
+
     warnings.filterwarnings("ignore")
 
     rescaled_mask_2d = rescale(
@@ -1194,14 +1302,37 @@ def native_index_for_slim_index_2d_from(
     Returns an array of shape [total_unmasked_pixels*sub_size] that maps every unmasked sub-pixel to its
     corresponding native 2D pixel using its (y,x) pixel indexes.
 
-    For example, for a sub-grid size of 2x2, if pixel [2,5] corresponds to the first pixel in the masked slim array:
+    For example, for the following ``Mask2D`` for ``sub_size=1``:
 
-    - The first sub-pixel in this pixel on the 1D array is sub_native_index_for_sub_slim_index_2d[4] = [2,5]
-    - The second sub-pixel in this pixel on the 1D array is sub_native_index_for_sub_slim_index_2d[5] = [2,6]
-    - The third sub-pixel in this pixel on the 1D array is sub_native_index_for_sub_slim_index_2d[5] = [3,5]
+    ::
+        [[True,  True,  True, True]
+         [True, False, False, True],
+         [True, False,  True, True],
+         [True,  True,  True, True]]
+
+    This has three unmasked (``False`` values) which have the ``slim`` indexes:
+
+    ::
+        [0, 1, 2]
+
+    The array ``native_index_for_slim_index_2d`` is therefore:
+
+    ::
+        [[1,1], [1,2], [2,1]]
+
+    For a ``Mask2D`` with ``sub_size=2`` each unmasked ``False`` entry is split into a sub-pixel of size 2x2 and
+    there are therefore 12 ``slim`` indexes:
+
+    ::
+        [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]
+
+    The array ``native_index_for_slim_index_2d`` is therefore:
+
+    ::
+        [[2,2], [2,3], [2,4], [2,5], [3,2], [3,3], [3,4], [3,5], [4,2], [4,3], [5,2], [5,3]]
 
     Parameters
-    -----------
+    ----------
     mask_2d
         A 2D array of bools, where `False` values are unmasked.
     sub_size
@@ -1250,7 +1381,7 @@ def mask_2d_neighbors_from(mask_2d: np.ndarray) -> np.ndarray:
     in each location.
 
     Parameters
-    -----------
+    ----------
     mask_2d
         A 2D array of bools, where `False` values are unmasked.
 
