@@ -2,11 +2,13 @@ import logging
 import numpy as np
 from typing import List
 
-from autoarray.inversion.linear_eqn.imaging.abstract import AbstractLEqImaging
+
+from autoarray.inversion.inversion.imaging.abstract import AbstractInversionImaging
+from autoarray.inversion.linear_obj.func_list import AbstractLinearObjFuncList
+from autoarray.inversion.pixelization.mappers.abstract import AbstractMapper
 
 from autoarray import exc
-from autoarray.inversion.linear_eqn import leq_util
-
+from autoarray.inversion.inversion.imaging import inversion_imaging_util
 
 logger = logging.getLogger(__name__)
 
@@ -22,9 +24,13 @@ class Preloads:
         relocated_grid=None,
         mapper_list=None,
         operated_mapping_matrix=None,
+        linear_func_operated_mapping_matrix_dict=None,
+        linear_func_weighted_mapping_vectors_dict=None,
+        linear_func_curvature_vectors_dict=None,
         curvature_matrix_preload=None,
         curvature_matrix_counts=None,
         curvature_matrix=None,
+        curvature_matrix_mapper_diag=None,
         regularization_matrix=None,
         log_det_regularization_matrix_term=None,
         traced_sparse_grids_list_of_planes=None,
@@ -38,9 +44,17 @@ class Preloads:
         self.relocated_grid = relocated_grid
         self.mapper_list = mapper_list
         self.operated_mapping_matrix = operated_mapping_matrix
+        self.linear_func_operated_mapping_matrix_dict = (
+            linear_func_operated_mapping_matrix_dict
+        )
+        self.linear_func_weighted_mapping_vectors_dict = (
+            linear_func_weighted_mapping_vectors_dict
+        )
+        self.linear_func_curvature_vectors_dict = linear_func_curvature_vectors_dict
         self.curvature_matrix_preload = curvature_matrix_preload
         self.curvature_matrix_counts = curvature_matrix_counts
         self.curvature_matrix = curvature_matrix
+        self.curvature_matrix_mapper_diag = curvature_matrix_mapper_diag
         self.regularization_matrix = regularization_matrix
         self.log_det_regularization_matrix_term = log_det_regularization_matrix_term
 
@@ -73,19 +87,23 @@ class Preloads:
         if fit_0.inversion is None:
             return
 
-        if not fit_0.inversion.has_mapper:
+        if not fit_0.inversion.has(cls=AbstractMapper):
             return
 
         if np.max(abs(fit_0.noise_map - fit_1.noise_map)) < 1e-8:
 
             logger.info("PRELOADS - Computing W-Tilde... May take a moment.")
 
-            from autoarray.dataset.imaging import WTildeImaging
+            from autoarray.dataset.imaging.w_tilde import WTildeImaging
 
-            preload, indexes, lengths = leq_util.w_tilde_curvature_preload_imaging_from(
+            (
+                preload,
+                indexes,
+                lengths,
+            ) = inversion_imaging_util.w_tilde_curvature_preload_imaging_from(
                 noise_map_native=fit_0.noise_map.native,
                 kernel_native=fit_0.dataset.psf.native,
-                native_index_for_slim_index=fit_0.dataset.mask.native_index_for_slim_index,
+                native_index_for_slim_index=fit_0.dataset.mask.derive_indexes.native_for_slim,
             )
 
             self.w_tilde = WTildeImaging(
@@ -123,20 +141,31 @@ class Preloads:
         if fit_0.inversion is None:
             return
 
-        if fit_0.inversion.total_mappers > 1 or fit_0.inversion.total_mappers == 0:
+        if (
+            fit_0.inversion.total(cls=AbstractMapper) > 1
+            or fit_0.inversion.total(cls=AbstractMapper) == 0
+        ):
             return
 
-        mapper_0 = fit_0.inversion.mapper_list[0]
-        mapper_1 = fit_1.inversion.mapper_list[0]
+        mapper_0 = fit_0.inversion.cls_list_from(cls=AbstractMapper)[0]
+        mapper_1 = fit_1.inversion.cls_list_from(cls=AbstractMapper)[0]
 
-        if mapper_0.source_grid_slim.shape[0] == mapper_1.source_grid_slim.shape[0]:
+        if (
+            mapper_0.source_plane_data_grid.shape[0]
+            == mapper_1.source_plane_data_grid.shape[0]
+        ):
 
             if (
-                np.max(abs(mapper_0.source_grid_slim - mapper_1.source_grid_slim))
+                np.max(
+                    abs(
+                        mapper_0.source_plane_data_grid
+                        - mapper_1.source_plane_data_grid
+                    )
+                )
                 < 1.0e-8
             ):
 
-                self.relocated_grid = mapper_0.source_grid_slim
+                self.relocated_grid = mapper_0.source_plane_data_grid
 
                 logger.info(
                     "PRELOADS - Relocated grid of pxielization preloaded for this model-fit."
@@ -144,7 +173,7 @@ class Preloads:
 
     def set_mapper_list(self, fit_0, fit_1):
         """
-        If the `MassProfile`'s and `Pixelization`'s in a model are fixed, the mapping of image-pixels to the
+        If the `MassProfile`'s and `Mesh`'s in a model are fixed, the mapping of image-pixels to the
         source-pixels does not change during the model-fit and the list of `Mapper`'s containing this information can
         be preloaded. This includes preloading the `mapping_matrix`.
 
@@ -167,14 +196,14 @@ class Preloads:
         if fit_0.inversion is None:
             return
 
-        if fit_0.inversion.total_mappers == 0:
+        if fit_0.inversion.total(cls=AbstractMapper) == 0:
             return
 
-        from autoarray.inversion.inversion.linear_operator import (
-            InversionLinearOperator,
+        from autoarray.inversion.inversion.interferometer.lop import (
+            InversionInterferometerMappingPyLops,
         )
 
-        if isinstance(fit_0.inversion, InversionLinearOperator):
+        if isinstance(fit_0.inversion, InversionInterferometerMappingPyLops):
             return
 
         inversion_0 = fit_0.inversion
@@ -184,7 +213,7 @@ class Preloads:
 
             if np.allclose(inversion_0.mapping_matrix, inversion_1.mapping_matrix):
 
-                self.mapper_list = inversion_0.mapper_list
+                self.mapper_list = inversion_0.cls_list_from(cls=AbstractMapper)
 
                 logger.info(
                     "PRELOADS - Mappers of planes preloaded for this model-fit."
@@ -192,7 +221,7 @@ class Preloads:
 
     def set_operated_mapping_matrix_with_preloads(self, fit_0, fit_1):
         """
-        If the `MassProfile`'s and `Pixelization`'s in a model are fixed, the mapping of image-pixels to the
+        If the `MassProfile`'s and `Mesh`'s in a model are fixed, the mapping of image-pixels to the
         source-pixels does not change during the model-fit and matrices used to perform the linear algebra in an
         inversion can be preloaded, which help efficiently construct the curvature matrix.
 
@@ -214,11 +243,11 @@ class Preloads:
         self.curvature_matrix_preload = None
         self.curvature_matrix_counts = None
 
-        from autoarray.inversion.inversion.linear_operator import (
-            InversionLinearOperator,
+        from autoarray.inversion.inversion.interferometer.lop import (
+            InversionInterferometerMappingPyLops,
         )
 
-        if isinstance(fit_0.inversion, InversionLinearOperator):
+        if isinstance(fit_0.inversion, InversionInterferometerMappingPyLops):
             return
 
         inversion_0 = fit_0.inversion
@@ -244,7 +273,7 @@ class Preloads:
 
                 self.operated_mapping_matrix = inversion_0.operated_mapping_matrix
 
-                if isinstance(inversion_0.leq, AbstractLEqImaging):
+                if isinstance(inversion_0, AbstractInversionImaging):
 
                     self.curvature_matrix_preload = (
                         inversion_0.curvature_matrix_preload
@@ -254,15 +283,106 @@ class Preloads:
                     ).astype("int")
 
                 logger.info(
-                    "PRELOADS - LEq linear algebra quantities preloaded for this model-fit."
+                    "PRELOADS - Inversion linear algebra quantities preloaded for this model-fit."
                 )
 
+    def set_linear_func_inversion_dicts(self, fit_0, fit_1):
+        """
+        If the `MassProfile`'s and `Mesh`'s in a model are fixed, the mapping of image-pixels to the
+        source-pixels does not change during the model-fit and matrices used to perform the linear algebra in an
+        inversion can be preloaded, which help efficiently construct the curvature matrix.
+
+        This function compares the operated mapping matrix of two fit's corresponding to two model instances, and
+        preloads the mapper if the mapping matrix of both fits are the same.
+
+        The preload is typically used in searches where only light profiles vary (e.g. when only the lens's light is
+        being fitted for).
+
+        Parameters
+        ----------
+        fit_0
+            The first fit corresponding to a model with a specific set of unit-values.
+        fit_1
+            The second fit corresponding to a model with a different set of unit-values.
+        """
+
+        self.linear_func_operated_mapping_matrix_dict = None
+        self.linear_func_weighted_mapping_vectors_dict = None
+        self.linear_func_curvature_vectors_dict = None
+
+        inversion_0 = fit_0.inversion
+        inversion_1 = fit_1.inversion
+
+        if inversion_0 is None:
+            return
+
+        if not inversion_0.has(cls=AbstractLinearObjFuncList):
+            return
+
+        if not hasattr(inversion_0, "linear_func_operated_mapping_matrix_dict"):
+            return
+
+        should_preload = False
+
+        for operated_mapping_matrix_0, operated_mapping_matrix_1 in zip(
+            inversion_0.linear_func_operated_mapping_matrix_dict.values(),
+            inversion_1.linear_func_operated_mapping_matrix_dict.values(),
+        ):
+
+            if (
+                np.max(abs(operated_mapping_matrix_0 - operated_mapping_matrix_1))
+                < 1e-8
+            ):
+
+                should_preload = True
+
+        if should_preload:
+
+            self.linear_func_operated_mapping_matrix_dict = (
+                inversion_0.linear_func_operated_mapping_matrix_dict
+            )
+
+            logger.info(
+                "PRELOADS - Inversion linear light profile operated mapping matrix preloaded for this model-fit."
+            )
+
+        should_preload = False
+
+        for weighted_mapping_vectors_0, weighted_mapping_vectors_1 in zip(
+            inversion_0.linear_func_weighted_mapping_vectors_dict.values(),
+            inversion_1.linear_func_weighted_mapping_vectors_dict.values(),
+        ):
+
+            if (
+                np.max(abs(weighted_mapping_vectors_0 - weighted_mapping_vectors_1))
+                < 1e-8
+            ):
+
+                should_preload = True
+
+        if should_preload:
+
+            self.linear_func_weighted_mapping_vectors_dict = (
+                inversion_0.linear_func_weighted_mapping_vectors_dict
+            )
+            self.linear_func_curvature_vectors_dict = (
+                inversion_0.linear_func_curvature_vectors_dict
+            )
+
+            logger.info(
+                "PRELOADS - Inversion linear light profile quantities preloaded for this model-fit."
+            )
+
     def set_curvature_matrix(self, fit_0, fit_1):
         """
-        If the `MassProfile`'s and `Pixelization`'s in a model are fixed, the mapping of image-pixels to the
-        source-pixels does not change during the model-fit and therefore its associated cruvature matrix is also
+        If the `MassProfile`'s and `Mesh`'s in a model are fixed, the mapping of image-pixels to the
+        source-pixels does not change during the model-fit and therefore its associated curvature matrix is also
         fixed, meaning the curvature matrix preloaded.
 
+        If linear ``LightProfiles``'s are included, the regions of the curvature matrix associatd with these
+        objects vary but the diagonals of the mapper do not change. In this case, the `curvature_matrix_mapper_diag`
+        is preloaded.
+
         This function compares the curvature matrix of two fit's corresponding to two model instances, and preloads
         this value if it is the same for both fits.
 
@@ -275,7 +395,9 @@ class Preloads:
         fit_1
             The second fit corresponding to a model with a different set of unit-values.
         """
+
         self.curvature_matrix = None
+        self.curvature_matrix_mapper_diag = None
 
         inversion_0 = fit_0.inversion
         inversion_1 = fit_1.inversion
@@ -283,6 +405,9 @@ class Preloads:
         if inversion_0 is None:
             return
 
+        if not hasattr(inversion_0, "_curvature_matrix_mapper_diag"):
+            return
+
         if inversion_0.curvature_matrix.shape == inversion_1.curvature_matrix.shape:
 
             if (
@@ -293,12 +418,32 @@ class Preloads:
                 self.curvature_matrix = inversion_0.curvature_matrix
 
                 logger.info(
-                    "PRELOADS - LEq Curvature Matrix preloaded for this model-fit."
+                    "PRELOADS - Inversion Curvature Matrix preloaded for this model-fit."
+                )
+
+                return
+
+            elif (
+                np.max(
+                    abs(
+                        inversion_0._curvature_matrix_mapper_diag
+                        - inversion_1._curvature_matrix_mapper_diag
+                    )
+                )
+                < 1e-8
+            ):
+
+                self.curvature_matrix_mapper_diag = (
+                    inversion_0._curvature_matrix_mapper_diag
+                )
+
+                logger.info(
+                    "PRELOADS - Inversion Curvature Matrix Mapper Diag preloaded for this model-fit."
                 )
 
     def set_regularization_matrix_and_term(self, fit_0, fit_1):
         """
-        If the `MassProfile`'s and `Pixelization`'s in a model are fixed, the mapping of image-pixels to the
+        If the `MassProfile`'s and `Mesh`'s in a model are fixed, the mapping of image-pixels to the
         source-pixels does not change during the model-fit and therefore its associated regularization matrices are
         also fixed, meaning the log determinant of the regularization matrix term of the Bayesian evidence can be
         preloaded.
@@ -325,7 +470,7 @@ class Preloads:
         if inversion_0 is None:
             return
 
-        if inversion_0.total_mappers == 0:
+        if inversion_0.total(cls=AbstractMapper) == 0:
             return
 
         if (
@@ -342,7 +487,7 @@ class Preloads:
             )
 
             logger.info(
-                "PRELOADS - LEq Log Det Regularization Matrix Term preloaded for this model-fit."
+                "PRELOADS - Inversion Log Det Regularization Matrix Term preloaded for this model-fit."
             )
 
     def check_via_fit(self, fit):
@@ -350,25 +495,75 @@ class Preloads:
         import copy
 
         settings_inversion = copy.deepcopy(fit.settings_inversion)
-        settings_inversion.use_curvature_matrix_preload = False
 
-        fom_with_preloads = fit.refit_with_new_preloads(
+        fit_with_preloads = fit.refit_with_new_preloads(
             preloads=self, settings_inversion=settings_inversion
-        ).figure_of_merit
+        )
 
-        fom_without_preloads = fit.refit_with_new_preloads(
-            preloads=self.__class__(use_w_tilde=False)
-        ).figure_of_merit
+        fit_without_preloads = fit.refit_with_new_preloads(
+            preloads=self.__class__(use_w_tilde=False),
+            settings_inversion=settings_inversion,
+        )
 
-        if abs(fom_with_preloads - fom_without_preloads) > 1.0e-4:
+        try:
 
-            raise exc.PreloadsException(
-                f"The log likelihood of fits using and not using preloads are not"
-                f"consistent, indicating preloading has gone wrong."
-                f"The likelihood values are {fom_with_preloads} (with preloads) and "
-                f"{fom_without_preloads} (without preloads)"
+            if (
+                abs(
+                    fit_with_preloads.figure_of_merit
+                    - fit_without_preloads.figure_of_merit
+                )
+                > 1.0e-4
+            ):
+
+                raise exc.PreloadsException(
+                    f"""
+                    The log likelihood of fits using and not using preloads are not consistent, indicating 
+                    preloading has gone wrong.
+
+
+                    The likelihood values are: 
+
+                    With Preloads: {fit_with_preloads.figure_of_merit}
+                    Without Preloads: {fit_without_preloads.figure_of_merit}
+                    """
+                )
+
+        except exc.InversionException:
+
+            data_vector_difference = np.max(
+                np.abs(
+                    fit_with_preloads.inversion.data_vector
+                    - fit_without_preloads.inversion.data_vector
+                )
             )
 
+            if data_vector_difference > 1.0e-4:
+                raise exc.PreloadsException(
+                    f"""
+                    The data vectors of fits using and not using preloads are not consistent, indicating 
+                    preloading has gone wrong.
+
+                    The maximum value a data vector absolute value difference is: {data_vector_difference} 
+                    """
+                )
+
+            curvature_reg_matrix_difference = np.max(
+                np.abs(
+                    fit_with_preloads.inversion.curvature_reg_matrix
+                    - fit_without_preloads.inversion.curvature_reg_matrix
+                )
+            )
+
+            if curvature_reg_matrix_difference > 1.0e-4:
+                raise exc.PreloadsException(
+                    f"""
+                    The curvature matrices of fits using and not using preloads are not consistent, indicating 
+                    preloading has gone wrong.
+
+                    The maximum value of a curvature matrix absolute value difference is: {curvature_reg_matrix_difference} 
+                    """
+                )
+
     def reset_all(self):
         """
         Reset all preloads, typically done at the end of a model-fit to save memory.
@@ -406,6 +601,9 @@ class Preloads:
             f"Curvature Matrix Sparse = {self.curvature_matrix_preload is not None}\n"
         ]
         line += [f"Curvature Matrix = {self.curvature_matrix is not None}\n"]
+        line += [
+            f"Curvature Matrix Mapper Diag = {self.curvature_matrix_mapper_diag is not None}\n"
+        ]
         line += [f"Regularization Matrix = {self.regularization_matrix is not None}\n"]
         line += [
             f"Log Det Regularization Matrix Term = {self.log_det_regularization_matrix_term is not None}\n"
