@@ -1,11 +1,17 @@
+from __future__ import annotations
 from abc import ABC
 from abc import abstractmethod
 import numpy as np
-from typing import List, Tuple, Union
+from typing import TYPE_CHECKING, List, Tuple, Union
+
+if TYPE_CHECKING:
+    from autoarray.structures.grids.uniform_1d import Grid1D
+    from autoarray.structures.grids.uniform_2d import Grid2D
 
 from autoarray.abstract_ndarray import AbstractNDArray
-
-from autoarray.structures.arrays import array_2d_util
+from autoarray.mask.derive.grid_2d import DeriveGrid2D
+from autoarray.mask.derive.indexes_2d import DeriveIndexes2D
+from autoarray.mask.derive.mask_2d import DeriveMask2D
 
 from autoarray import exc
 
@@ -24,11 +30,20 @@ class Structure(AbstractNDArray, ABC):
         """
 
     @property
-    @abstractmethod
-    def native(self) -> "Structure":
-        """
-        Returns the data structure in its `native` format which contains all unmaksed values to the native dimensions.
-        """
+    def geometry(self):
+        return self.mask.geometry
+
+    @property
+    def derive_grid(self) -> DeriveGrid2D:
+        return self.mask.derive_grid
+
+    @property
+    def derive_indexes(self) -> DeriveIndexes2D:
+        return self.mask.derive_indexes
+
+    @property
+    def derive_mask(self) -> DeriveMask2D:
+        return self.mask.derive_mask
 
     @property
     def shape_slim(self) -> int:
@@ -76,8 +91,8 @@ class Structure(AbstractNDArray, ABC):
         return self.mask.sub_size
 
     @property
-    def unmasked_grid(self) -> Union["Grid1D", "Grid2D"]:
-        return self.mask.unmasked_grid_sub_1
+    def unmasked_grid(self) -> Union[Grid1D, Grid2D]:
+        return self.mask.derive_grid.all_false_sub_1
 
     @property
     def total_pixels(self) -> int:
@@ -89,17 +104,5 @@ class Structure(AbstractNDArray, ABC):
     def structure_2d_from(self, result: np.ndarray) -> "Structure":
         raise NotImplementedError
 
-    def output_to_fits(self, file_path: str, overwrite: bool = False):
-        """
-        Output the grid to a .fits file.
-
-        Parameters
-        ----------
-        file_path
-            The path the file is output to, including the filename and the .fits extension, e.g. '/path/to/filename.fits'
-        overwrite
-            If a file already exists at the path, if overwrite=True it is overwritten else an error is raised.
-        """
-        array_2d_util.numpy_array_2d_to_fits(
-            array_2d=self.native, file_path=file_path, overwrite=overwrite
-        )
+    def trimmed_after_convolution_from(self, kernel_shape) -> "Structure":
+        raise NotImplementedError
