@@ -1,5 +1,9 @@
+from __future__ import annotations
 import numpy as np
-from typing import List, Optional, Tuple, Union
+from typing import TYPE_CHECKING, List, Optional, Tuple, Union
+
+if TYPE_CHECKING:
+    from autoarray.mask.mask_2d import Mask2D
 
 from autoarray import exc
 from autoarray.structures.arrays import array_2d_util
@@ -9,6 +13,28 @@ from autoarray.mask import mask_2d_util
 from autoarray import type as ty
 
 
+def check_grid_slim(grid, shape_native):
+
+    if shape_native is None:
+        raise exc.GridException(
+            f"""
+            The input grid is not in its native shape (an ndarray / list of shape [total_y_pixels, total_x_pixels, 2])
+            and the shape_native parameter has not been input the Grid2D function.
+
+            Either change the input array to be its native shape or input its shape_native input the function.
+
+            The shape of the input array is {grid.shape}
+            """
+        )
+
+    if shape_native and len(shape_native) != 2:
+        raise exc.GridException(
+            """
+            The input shape_native parameter is not a tuple of type (int, int).
+            """
+        )
+
+
 def convert_grid(grid: Union[np.ndarray, List]) -> np.ndarray:
 
     if type(grid) is list:
@@ -27,25 +53,41 @@ def check_grid_2d(grid_2d: np.ndarray):
         raise exc.GridException("The dimensions of the input grid array is not 2 or 3")
 
 
-def check_grid_2d_and_mask_2d(grid_2d: np.ndarray, mask_2d: "Mask2D"):
+def check_grid_2d_and_mask_2d(grid_2d: np.ndarray, mask_2d: Mask2D):
+
     if len(grid_2d.shape) == 2:
 
         if grid_2d.shape[0] != mask_2d.sub_pixels_in_mask:
             raise exc.GridException(
-                "The input 1D grid does not have the same number of entries as sub-pixels in"
-                "the mask."
+                f"""
+                The input 2D grid does not have the same number of values as sub-pixels in
+                the mask.
+                
+                The shape of the input grid_2d is {grid_2d.shape}.
+                The mask shape_native is {mask_2d.shape_native}.
+                The mask number of sub-pixels is {mask_2d.sub_pixels_in_mask}. 
+                The mask sub size is {mask_2d.sub_size}).
+                """
             )
 
     elif len(grid_2d.shape) == 3:
 
         if (grid_2d.shape[0], grid_2d.shape[1]) != mask_2d.sub_shape_native:
             raise exc.GridException(
-                "The input grid is 2D but not the same dimensions as the sub-mask "
-                "(e.g. the mask 2D shape multipled by its sub size.)"
+                f"""
+                The input 2D grid is not the same dimensions as the sub-mask
+                (e.g. the mask 2D shape multipled by its sub size.)
+
+                The shape of the input grid_2d is {grid_2d.shape}.
+                The sub_shape_native of the mask is {mask_2d.sub_shape_native} (the 
+                mask sub size is {mask_2d.sub_size}).
+                """
             )
 
 
-def convert_grid_2d(grid_2d: Union[np.ndarray, List], mask_2d: "Mask2D") -> np.ndarray:
+def convert_grid_2d(
+    grid_2d: Union[np.ndarray, List], mask_2d: Mask2D, store_native: bool = False
+) -> np.ndarray:
     """
     The `manual` classmethods in the Grid2D object take as input a list or ndarray which is returned as a Grid2D.
 
@@ -64,18 +106,37 @@ def convert_grid_2d(grid_2d: Union[np.ndarray, List], mask_2d: "Mask2D") -> np.n
         The input (y,x) grid of coordinates which is converted to an ndarray if it is a list.
     mask_2d
         The mask of the output Array2D.
+    store_native
+        If True, the ndarray is stored in its native format [total_y_pixels, total_x_pixels, 2]. This avoids
+        mapping large data arrays to and from the slim / native formats, which can be a computational bottleneck.
     """
 
     grid_2d = convert_grid(grid=grid_2d)
 
-    return convert_grid_2d_to_slim(grid_2d=grid_2d, mask_2d=mask_2d)
+    check_grid_2d_and_mask_2d(grid_2d=grid_2d, mask_2d=mask_2d)
+
+    is_native = len(grid_2d.shape) == 3
+
+    if is_native:
+        grid_2d[:, :, 0] *= np.invert(mask_2d.derive_mask.sub)
+        grid_2d[:, :, 1] *= np.invert(mask_2d.derive_mask.sub)
+
+    if is_native == store_native:
+        return grid_2d
+    elif not store_native:
+        return grid_2d_slim_from(
+            grid_2d_native=grid_2d, mask=mask_2d, sub_size=mask_2d.sub_size
+        )
+    return grid_2d_native_from(
+        grid_2d_slim=grid_2d, mask_2d=mask_2d, sub_size=mask_2d.sub_size
+    )
 
 
 def convert_grid_2d_to_slim(
-    grid_2d: Union[np.ndarray, List], mask_2d: "Mask2D"
+    grid_2d: Union[np.ndarray, List], mask_2d: Mask2D
 ) -> np.ndarray:
     """
-    he `manual` classmethods in the Grid2D object take as input a list or ndarray which is returned as a Grid2D. 
+    he `manual` classmethods in the Grid2D object take as input a list or ndarray which is returned as a Grid2D.
 
     This function checks the dimensions of the input `grid_2d` and maps it to its `slim` representation.
 
@@ -96,10 +157,10 @@ def convert_grid_2d_to_slim(
 
 
 def convert_grid_2d_to_native(
-    grid_2d: Union[np.ndarray, List], mask_2d: "Mask2D"
+    grid_2d: Union[np.ndarray, List], mask_2d: Mask2D
 ) -> np.ndarray:
     """
-    he `manual` classmethods in the Grid2D object take as input a list or ndarray which is returned as a Grid2D. 
+    he `manual` classmethods in the Grid2D object take as input a list or ndarray which is returned as a Grid2D.
 
     This function checks the dimensions of the input `grid_2d` and maps it to its `native` representation.
 
@@ -375,7 +436,7 @@ def grid_2d_via_shape_native_from(
 
 
 @numba_util.jit()
-def radial_projected_shape_slim_from(
+def _radial_projected_shape_slim_from(
     extent: np.ndarray,
     centre: Tuple[float, float],
     pixel_scales: ty.PixelScales,
@@ -554,321 +615,6 @@ def grid_scaled_2d_slim_radial_projected_from(
     return grid_scaled_2d_slim_radii
 
 
-@numba_util.jit()
-def grid_pixels_2d_slim_from(
-    grid_scaled_2d_slim: np.ndarray,
-    shape_native: Tuple[int, int],
-    pixel_scales: ty.PixelScales,
-    origin: Tuple[float, float] = (0.0, 0.0),
-) -> np.ndarray:
-    """
-    Convert a slimmed grid of 2d (y,x) scaled coordinates to a slimmed grid of 2d (y,x) pixel coordinate values. Pixel
-    coordinates are returned as floats such that they include the decimal offset from each pixel's top-left corner
-    relative to the input scaled coordinate.
-
-    The input and output grids are both slimmed and therefore shape (total_pixels, 2).
-
-    The pixel coordinate origin is at the top left corner of the grid, such that the pixel [0,0] corresponds to
-    the highest (most positive) y scaled coordinate and lowest (most negative) x scaled coordinate on the gird.
-
-    The scaled grid is defined by an origin and coordinates are shifted to this origin before computing their
-    1D grid pixel coordinate values.
-
-    Parameters
-    ----------
-    grid_scaled_2d_slim: np.ndarray
-        The slimmed grid of 2D (y,x) coordinates in scaled units which are converted to pixel value coordinates.
-    shape_native
-        The (y,x) shape of the original 2D array the scaled coordinates were computed on.
-    pixel_scales
-        The (y,x) scaled units to pixel units conversion factor of the original 2D array.
-    origin
-        The (y,x) origin of the grid, which the scaled grid is shifted to.
-
-    Returns
-    -------
-    ndarray
-        A slimmed grid of 2D (y,x) pixel-value coordinates with dimensions (total_pixels, 2).
-
-    Examples
-    --------
-    grid_scaled_2d_slim = np.array([[1.0, 1.0], [2.0, 2.0], [3.0, 3.0], [4.0, 4.0]])
-    grid_pixels_2d_slim = grid_scaled_2d_slim_from(grid_scaled_2d_slim=grid_scaled_2d_slim, shape=(2,2),
-                                                           pixel_scales=(0.5, 0.5), origin=(0.0, 0.0))
-    """
-
-    grid_pixels_2d_slim = np.zeros((grid_scaled_2d_slim.shape[0], 2))
-
-    centres_scaled = geometry_util.central_scaled_coordinate_2d_from(
-        shape_native=shape_native, pixel_scales=pixel_scales, origin=origin
-    )
-
-    for slim_index in range(grid_scaled_2d_slim.shape[0]):
-
-        grid_pixels_2d_slim[slim_index, 0] = (
-            (-grid_scaled_2d_slim[slim_index, 0] / pixel_scales[0])
-            + centres_scaled[0]
-            + 0.5
-        )
-        grid_pixels_2d_slim[slim_index, 1] = (
-            (grid_scaled_2d_slim[slim_index, 1] / pixel_scales[1])
-            + centres_scaled[1]
-            + 0.5
-        )
-
-    return grid_pixels_2d_slim
-
-
-@numba_util.jit()
-def grid_pixel_centres_2d_slim_from(
-    grid_scaled_2d_slim: np.ndarray,
-    shape_native: Tuple[int, int],
-    pixel_scales: ty.PixelScales,
-    origin: Tuple[float, float] = (0.0, 0.0),
-) -> np.ndarray:
-    """
-    Convert a slimmed grid of 2D (y,x) scaled coordinates to a slimmed grid of 2D (y,x) pixel values. Pixel coordinates
-    are returned as integers such that they map directly to the pixel they are contained within.
-
-    The input and output grids are both slimmed and therefore shape (total_pixels, 2).
-
-    The pixel coordinate origin is at the top left corner of the grid, such that the pixel [0,0] corresponds to
-    the highest (most positive) y scaled coordinate and lowest (most negative) x scaled coordinate on the gird.
-
-    The scaled coordinate grid is defined by the class attribute origin, and coordinates are shifted to this
-    origin before computing their 1D grid pixel indexes.
-
-    Parameters
-    ----------
-    grid_scaled_2d_slim: np.ndarray
-        The slimmed grid of 2D (y,x) coordinates in scaled units which is converted to pixel indexes.
-    shape_native
-        The (y,x) shape of the original 2D array the scaled coordinates were computed on.
-    pixel_scales
-        The (y,x) scaled units to pixel units conversion factor of the original 2D array.
-    origin
-        The (y,x) origin of the grid, which the scaled grid is shifted
-
-    Returns
-    -------
-    ndarray
-        A slimmed grid of 2D (y,x) pixel indexes with dimensions (total_pixels, 2).
-
-    Examples
-    --------
-    grid_scaled_2d_slim = np.array([[1.0, 1.0], [2.0, 2.0], [3.0, 3.0], [4.0, 4.0]])
-    grid_pixels_2d_slim = grid_scaled_2d_slim_from(grid_scaled_2d_slim=grid_scaled_2d_slim, shape=(2,2),
-                                                           pixel_scales=(0.5, 0.5), origin=(0.0, 0.0))
-    """
-
-    grid_pixels_2d_slim = np.zeros((grid_scaled_2d_slim.shape[0], 2))
-
-    centres_scaled = geometry_util.central_scaled_coordinate_2d_from(
-        shape_native=shape_native, pixel_scales=pixel_scales, origin=origin
-    )
-
-    for slim_index in range(grid_scaled_2d_slim.shape[0]):
-
-        grid_pixels_2d_slim[slim_index, 0] = int(
-            (-grid_scaled_2d_slim[slim_index, 0] / pixel_scales[0])
-            + centres_scaled[0]
-            + 0.5
-        )
-        grid_pixels_2d_slim[slim_index, 1] = int(
-            (grid_scaled_2d_slim[slim_index, 1] / pixel_scales[1])
-            + centres_scaled[1]
-            + 0.5
-        )
-
-    return grid_pixels_2d_slim
-
-
-@numba_util.jit()
-def grid_pixel_indexes_2d_slim_from(
-    grid_scaled_2d_slim: np.ndarray,
-    shape_native: Tuple[int, int],
-    pixel_scales: ty.PixelScales,
-    origin: Tuple[float, float] = (0.0, 0.0),
-) -> np.ndarray:
-    """
-    Convert a slimmed grid of 2D (y,x) scaled coordinates to a slimmed grid of pixel indexes. Pixel coordinates are
-    returned as integers such that they are the pixel from the top-left of the 2D grid going rights and then downwards.
-
-    The input and output grids are both slimmed and have shapes (total_pixels, 2) and (total_pixels,).
-
-    For example:
-
-    The pixel at the top-left, whose native index is [0,0], corresponds to slimmed pixel index 0.
-    The fifth pixel on the top row, whose native index is [0,5], corresponds to slimmed pixel index 4.
-    The first pixel on the second row, whose native index is [0,1], has slimmed pixel index 10 if a row has 10 pixels.
-
-    The scaled coordinate grid is defined by the class attribute origin, and coordinates are shifted to this
-    origin before computing their 1D grid pixel indexes.
-
-    The input and output grids are both of shape (total_pixels, 2).
-
-    Parameters
-    ----------
-    grid_scaled_2d_slim: np.ndarray
-        The slimmed grid of 2D (y,x) coordinates in scaled units which is converted to slimmed pixel indexes.
-    shape_native
-        The (y,x) shape of the original 2D array the scaled coordinates were computed on.
-    pixel_scales
-        The (y,x) scaled units to pixel units conversion factor of the original 2D array.
-    origin
-        The (y,x) origin of the grid, which the scaled grid is shifted.
-
-    Returns
-    -------
-    ndarray
-        A grid of slimmed pixel indexes with dimensions (total_pixels,).
-
-    Examples
-    --------
-    grid_scaled_2d_slim = np.array([[1.0, 1.0], [2.0, 2.0], [3.0, 3.0], [4.0, 4.0]])
-    grid_pixel_indexes_2d_slim = grid_pixel_indexes_2d_slim_from(grid_scaled_2d_slim=grid_scaled_2d_slim, shape=(2,2),
-                                                           pixel_scales=(0.5, 0.5), origin=(0.0, 0.0))
-    """
-
-    grid_pixels_2d_slim = grid_pixel_centres_2d_slim_from(
-        grid_scaled_2d_slim=grid_scaled_2d_slim,
-        shape_native=shape_native,
-        pixel_scales=pixel_scales,
-        origin=origin,
-    )
-
-    grid_pixel_indexes_2d_slim = np.zeros(grid_pixels_2d_slim.shape[0])
-
-    for slim_index in range(grid_pixels_2d_slim.shape[0]):
-
-        grid_pixel_indexes_2d_slim[slim_index] = int(
-            grid_pixels_2d_slim[slim_index, 0] * shape_native[1]
-            + grid_pixels_2d_slim[slim_index, 1]
-        )
-
-    return grid_pixel_indexes_2d_slim
-
-
-@numba_util.jit()
-def grid_scaled_2d_slim_from(
-    grid_pixels_2d_slim: np.ndarray,
-    shape_native: Tuple[int, int],
-    pixel_scales: ty.PixelScales,
-    origin: Tuple[float, float] = (0.0, 0.0),
-) -> np.ndarray:
-    """
-    Convert a slimmed grid of 2D (y,x) pixel coordinates to a slimmed grid of 2D (y,x) scaled values.
-
-    The input and output grids are both slimmed and therefore shape (total_pixels, 2).
-
-    The pixel coordinate origin is at the top left corner of the grid, such that the pixel [0,0] corresponds to
-    the highest (most positive) y scaled coordinate and lowest (most negative) x scaled coordinate on the gird.
-
-    The scaled coordinate origin is defined by the class attribute origin, and coordinates are shifted to this
-    origin after computing their values from the 1D grid pixel indexes.
-
-    Parameters
-    ----------
-    grid_pixels_2d_slim: np.ndarray
-        The slimmed grid of (y,x) coordinates in pixel values which is converted to scaled coordinates.
-    shape_native
-        The (y,x) shape of the original 2D array the scaled coordinates were computed on.
-    pixel_scales
-        The (y,x) scaled units to pixel units conversion factor of the original 2D array.
-    origin
-        The (y,x) origin of the grid, which the scaled grid is shifted.
-
-    Returns
-    -------
-    ndarray
-        A slimmed grid of 2d scaled coordinates with dimensions (total_pixels, 2).
-
-    Examples
-    --------
-    grid_pixels_2d_slim = np.array([[0,0], [0,1], [1,0], [1,1])
-    grid_pixels_2d_slim = grid_scaled_2d_slim_from(grid_pixels_2d_slim=grid_pixels_2d_slim, shape=(2,2),
-                                                           pixel_scales=(0.5, 0.5), origin=(0.0, 0.0))
-    """
-
-    grid_scaled_2d_slim = np.zeros((grid_pixels_2d_slim.shape[0], 2))
-
-    centres_scaled = geometry_util.central_scaled_coordinate_2d_from(
-        shape_native=shape_native, pixel_scales=pixel_scales, origin=origin
-    )
-
-    for slim_index in range(grid_scaled_2d_slim.shape[0]):
-
-        grid_scaled_2d_slim[slim_index, 0] = (
-            -(grid_pixels_2d_slim[slim_index, 0] - centres_scaled[0] - 0.5)
-            * pixel_scales[0]
-        )
-        grid_scaled_2d_slim[slim_index, 1] = (
-            grid_pixels_2d_slim[slim_index, 1] - centres_scaled[1] - 0.5
-        ) * pixel_scales[1]
-
-    return grid_scaled_2d_slim
-
-
-@numba_util.jit()
-def grid_pixel_centres_2d_from(
-    grid_scaled_2d: np.ndarray,
-    shape_native: Tuple[int, int],
-    pixel_scales: ty.PixelScales,
-    origin: Tuple[float, float] = (0.0, 0.0),
-) -> np.ndarray:
-    """
-    Convert a native grid of 2D (y,x) scaled coordinates to a native grid of 2D (y,x) pixel values. Pixel coordinates
-    are returned as integers such that they map directly to the pixel they are contained within.
-
-    The input and output grids are both native resolution and therefore have shape (y_pixels, x_pixels, 2).
-
-    The pixel coordinate origin is at the top left corner of the grid, such that the pixel [0,0] corresponds to
-    the highest (most positive) y scaled coordinate and lowest (most negative) x scaled coordinate on the gird.
-
-    The scaled coordinate grid is defined by the class attribute origin, and coordinates are shifted to this
-    origin before computing their 1D grid pixel indexes.
-
-    Parameters
-    ----------
-    grid_scaled_2d: np.ndarray
-        The native grid of 2D (y,x) coordinates in scaled units which is converted to pixel indexes.
-    shape_native
-        The (y,x) shape of the original 2D array the scaled coordinates were computed on.
-    pixel_scales
-        The (y,x) scaled units to pixel units conversion factor of the original 2D array.
-    origin
-        The (y,x) origin of the grid, which the scaled grid is shifted
-
-    Returns
-    -------
-    ndarray
-        A native grid of 2D (y,x) pixel indexes with dimensions (y_pixels, x_pixels, 2).
-
-    Examples
-    --------
-    grid_scaled_2d = np.array([[1.0, 1.0], [2.0, 2.0], [3.0, 3.0], [4.0, 4.0]])
-    grid_pixel_centres_2d = grid_pixel_centres_2d_from(grid_scaled_2d=grid_scaled_2d, shape=(2,2),
-                                                           pixel_scales=(0.5, 0.5), origin=(0.0, 0.0))
-    """
-
-    grid_pixels_2d = np.zeros((grid_scaled_2d.shape[0], grid_scaled_2d.shape[1], 2))
-
-    centres_scaled = geometry_util.central_scaled_coordinate_2d_from(
-        shape_native=shape_native, pixel_scales=pixel_scales, origin=origin
-    )
-
-    for y in range(grid_scaled_2d.shape[0]):
-        for x in range(grid_scaled_2d.shape[1]):
-            grid_pixels_2d[y, x, 0] = int(
-                (-grid_scaled_2d[y, x, 0] / pixel_scales[0]) + centres_scaled[0] + 0.5
-            )
-            grid_pixels_2d[y, x, 1] = int(
-                (grid_scaled_2d[y, x, 1] / pixel_scales[1]) + centres_scaled[1] + 0.5
-            )
-
-    return grid_pixels_2d
-
-
 @numba_util.jit()
 def relocated_grid_via_jit_from(grid, border_grid):
     """
@@ -1061,7 +807,7 @@ def grid_2d_slim_upscaled_from(
     """
 
     grid_2d_slim_upscaled = np.zeros(
-        shape=(grid_slim.shape[0] * upscale_factor ** 2, 2)
+        shape=(grid_slim.shape[0] * upscale_factor**2, 2)
     )
 
     upscale_index = 0
@@ -1107,7 +853,7 @@ def grid_2d_of_points_within_radius(
     for i in range(len(grid_2d[:, 0])):
         if (grid_2d[i, 0] - centre[0]) ** 2 + (
             grid_2d[i, 1] - centre[1]
-        ) ** 2 > radius ** 2:
+        ) ** 2 > radius**2:
             y_inside.append(grid_2d[i, 0])
             x_inside.append(grid_2d[i, 1])
 
